/* ====
   ==== FlashTris Game, (c) 1999-2001 by FlashCode (Sebastien Helleu)
   ====
   ====    web: http://flashcode.free.fr
   ==== e-mail: flashcode@free.fr
   ====
   ==== effet.c: effets graphiques
   ====

   ==== This file is part of FlashTris.
   ====
   ==== FlashTris is free software; you can redistribute it and/or modify
   ==== it under the terms of the GNU General Public License as published by
   ==== the Free Software Foundation; either version 2 of the License, or
   ==== (at your option) any later version.
   ====
   ==== FlashTris is distributed in the hope that it will be useful,
   ==== but WITHOUT ANY WARRANTY; without even the implied warranty of
   ==== MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   ==== GNU General Public License for more details.
   ====
   ==== You should have received a copy of the GNU General Public License
   ==== along with FlashTris; if not, write to the Free Software
   ==== Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Auteur  : FlashCode <flashcode@free.fr>
Creation: 1999/06/21

2001/09/06: FlashCode:
    - ajout de la licence
*/


#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>


#include "ftris.h"
#include "effet.h"
#include "menu_gfx.h"
#include "game.h"
#include "video.h"


/*
 * memcpy_rapide: accelere le memcpy en recopiant les octets 4 par 4
 */

void memcpy_rapide(unsigned char *cible, unsigned char *source, int nb_dwords)
{
    __asm__ __volatile__ ("
        movl %0, %%edi\n
        movl %1, %%esi\n
        movl %2, %%ecx\n
        rep\n
        movsl"
        :
        : "g" (cible), "g" (source), "g" (nb_dwords)
        : "ecx", "esi", "edi"
    );
}

/*
 * memset_rapide: accelere le memset en stockant un double mot
 */

/*void memset_rapide(unsigned char *cible, int valeur, int nb_dwords)
{
    __asm__ __volatile__ ("
        movl %0, %%edi\n
        movl %1, %%eax\n
        movl %2, %%ecx\n
        rep\n
        stosd"
        :
        : "g" (cible), "g" (valeur), "g" (nb_dwords)
        : "eax", "ecx", "edi"
    );
}*/

/*
 * assombrit_pixel: assombrit un pixel de l'ecran
 *                  (x,y):coordonnees
 *                  coul: numero de couleur du pixel le plus sombre
 *                  coul0: couleur pour remplacer un pixel noir de couleur 0
 *                         peut valoir 0 ou la meme couleur que le param 'coul'
 *                  facteur:plus il est grand, plus le pixel est assombri (1..10)
 */

void assombrit_pixel(int no_vscr_source, int no_vscr_cible,
                     int x, int y, int coul, int coul0, int facteur)
{
    int     i;

    #ifndef MSDOS
    /*  retirer ca !!! */
    if (X_Window)
        return;
    #endif
    /*i = source[x+y*scr_nbx];*/
    i = getpixel_ofs(no_vscr_source, x+y*scr_nbx);
    if (i != 0)
        putpixel_ofs(no_vscr_cible, x+y*scr_nbx,
            coul-(((palette[i*3]+palette[i*3+1]+palette[i*3+2])/3)/facteur));
    else
        putpixel_ofs(no_vscr_cible, x+y*scr_nbx, coul0);
}

/*
 * zoom: zoom arriere sur une image
 *       s_lx, s_ly: largeur, hauteur de l'image initiale
 *       c_lx, c_ly: largeur, hauteur de l'imag finale
 *       c_x, c_y  : coordonnees de l'image finale dans la cible
 */

void zoom(int no_vscr_source, int s_lx, int s_ly,
          int no_vscr_cible, int c_lx, int c_ly, int c_x, int c_y)
{
    int     ptr_source, ptr_cible, ptr_cible_apres;
    int     x, x2, y, y2;
    int     inc_x, inc_y;

    ptr_source = 0;
    ptr_cible = (c_y*scr_nbx)+c_x;
    y = 0; y2 = 0;
    inc_x = (s_lx-c_lx);
    inc_y = (s_ly-c_ly);
    while (y2 < s_ly)
    {
        y += inc_y;
        if (y >= 199)
        {
            y -= 199;
            ptr_source += scr_nbx;
        }
        else
        {
            x = 0; x2 = 0;
            ptr_cible_apres = ptr_cible+scr_nbx;
            while (x2 < s_lx)
            {
                x += inc_x;
                if (x >= 319)
                {
                    x -= 319;
                    ptr_source++;
                }
                else
                {
                    putpixel_ofs(no_vscr_cible, ptr_cible,
                        getpixel_ofs(no_vscr_source, ptr_source));
                    ptr_source++;
                    ptr_cible++;
                }
                x2++;
            }
            ptr_source += (scr_nbx-s_lx);
            ptr_cible = ptr_cible_apres;
        }
        y2++;
    }
}

/*
 * make_effect_debut: effet 0: fait apparaitre le menu en "eloignant" l'image de jeu
 *                    effet 1: fait apparaitre le menu en "eloignant" l'image de jeu et
 *                             en faisant apparaitre le menu des bords
 *                    effet 2: fait apparaitre le menu en "eloignant" l'image de jeu et
 *                             en faisant apparaitre le menu des bords (du centre vers l'exterieur)
 */

void make_effect_debut(int no_vscr_image, int no_effet, int nb_images,
                       int vscr_no_debut)
{
    int             i, j, x1, y1;

    /*  */
    #ifndef MSDOS
    if (X_Window)
        return;
    #endif
    if (vscr_no_debut+nb_images-1 >= VSCR_DERNIER)
    {
        close_grafx();
        fprintf(stderr, "FlashTris FATAL CRASH: image #%d + %d frames + 1 > # buffers (=%d)\n",
            vscr_no_debut, nb_images, VSCR_DERNIER);
        fprintf(stderr, REPORT_BUG);
        close_sb16();
        exit(1);
    }
    copy_s2v(VSCR_DERNIER);
    for (i = 0; i < nbj; i++)
        efface_piece_preview(VSCR_DERNIER, i, &piece[i]);

    /*printf("vscr_dernier=%d    vscr_no_debut=%d --> %d (nb_images=%d)\n",
        VSCR_DERNIER, vscr_no_debut, vscr_no_debut+nb_images-1, nb_images);*/

    switch (no_effet)
    {
        case 0:
            for (i = 0; i < nb_images; i++)
            {
                /*memcpy_rapide(vscreen[vscr_no_debut+i], vscreen[VSCR_DERNIER], 16000);*/
                copy_v2v(VSCR_DERNIER, vscr_no_debut+i);
                x1 = (96*(i+1))/nb_images;
                y1 = (60*(i+1))/nb_images;
                zoom(no_vscr_image, scr_nbx, scr_nby, vscr_no_debut+i,
                    scr_nbx-(x1*2), scr_nby-(y1*2), x1, y1);
            }
            break;
        case 1:
            for (i = 0; i < nb_images; i++)
            {
                /*memset(vscreen[vscr_no_debut+i], 0, 64000);*/
                clear_vscreen(vscr_no_debut+i);
                x1 = (96*(i+1))/nb_images;
                y1 = (60*(i+1))/nb_images;
                zoom(no_vscr_image, scr_nbx, scr_nby, vscr_no_debut+i,
                    scr_nbx-(x1*2), scr_nby-(y1*2), x1, y1);
                for (j = 0; j < 140; j++)
                {
                    /*memcpy(vscreen[vscr_no_debut+i]+j*scr_nbx, vscreen[VSCR_DERNIER]+j*scr_nbx+96-x1, x1);*/
                    copy_v2v_xy_xy(VSCR_DERNIER, vscr_no_debut+i, 96-x1, j, x1, 1, 0, j);
                    /*memcpy(vscreen[vscr_no_debut+i]+(j+60)*scr_nbx+224+96-x1, vscreen[VSCR_DERNIER]+(j+60)*scr_nbx+224, x1);*/
                    copy_v2v_xy_xy(VSCR_DERNIER, vscr_no_debut+i, 224, j+60, x1, 1, 224+96-x1, j+60);
                }
                for (j = 0; j < y1; j++)
                {
                    /*memcpy(vscreen[vscr_no_debut+i]+96+j*scr_nbx, vscreen[VSCR_DERNIER]+96+(60-y1+j)*scr_nbx, 224);*/
                    copy_v2v_xy_xy(VSCR_DERNIER, vscr_no_debut+i, 96, 60-y1+j, 224, 1, 96, j);
                    /*memcpy(vscreen[vscr_no_debut+i]+199*scr_nbx-j*scr_nbx, vscreen[VSCR_DERNIER]+140*scr_nbx+(y1-j-1)*scr_nbx, 224);*/
                    copy_v2v_xy_xy(VSCR_DERNIER, vscr_no_debut+i, 0, 140+y1-j-1, 224, 1, 0, 199-j);
                }
            }
            break;
        case 2:
            for (i = 0; i < nb_images; i++)
            {
                /*memset(vscreen[vscr_no_debut+i], 0, 64000);*/
                clear_vscreen(vscr_no_debut+i);
                x1 = (96*(i+1))/nb_images;
                y1 = (60*(i+1))/nb_images;
                for (j = 0; j < 140; j++)
                {
                    /*memcpy(vscreen[vscr_no_debut+i]+j*scr_nbx+96-x1, vscreen[VSCR_DERNIER]+j*scr_nbx, x1);*/
                    copy_v2v_xy_xy(VSCR_DERNIER, vscr_no_debut+i, 0, j, x1, 1, 96-x1, j);
                    /*memcpy(vscreen[vscr_no_debut+i]+(j+60)*scr_nbx+224, vscreen[VSCR_DERNIER]+(j+60)*scr_nbx+224+96-x1, x1);*/
                    copy_v2v_xy_xy(VSCR_DERNIER, vscr_no_debut+i, 224+96-x1, j+60, x1, 1, 224, j+60);
                }
                for (j = 0; j < y1; j++)
                {
                    /*memcpy(vscreen[vscr_no_debut+i]+96+(60-y1+j)*scr_nbx, vscreen[VSCR_DERNIER]+96+j*scr_nbx, 224);*/
                    copy_v2v_xy_xy(VSCR_DERNIER, vscr_no_debut+i, 96, j, 224, 1, 96, 60-y1+j);
                    /*memcpy(vscreen[vscr_no_debut+i]+140*scr_nbx+(y1-j-1)*scr_nbx, vscreen[VSCR_DERNIER]+199*scr_nbx-j*scr_nbx, 224);*/
                    copy_v2v_xy_xy(VSCR_DERNIER, vscr_no_debut+i, 0, 199-j, 224, 1, 0, 140+y1-j-1);
                }
                zoom(no_vscr_image, scr_nbx, scr_nby, vscr_no_debut+i, scr_nbx-(x1*2), scr_nby-(y1*2), x1, y1);
            }
            break;
    }

    for (i = nb_images-1; i >= 0; i--)
    {
        waitretrace();
        copy_v2s(vscr_no_debut+i);
    }
    waitretrace();
    copy_v2s(no_vscr_image);
}

/*
 * make_effect_fin: effet inverse de make_effect_debut (appele quand on retourne au menu)
 */

void make_effect_fin(int nb_images, int vscr_no_debut)
{
    int     i;

    for (i = 0; i < nb_images; i++)
    {
        waitretrace();
        copy_v2s(vscr_no_debut+i);
    }

    waitretrace();
    copy_v2s(VSCR_DERNIER);
    /*affiche_spirale(vscreen_effect, 20, 20, 0, 0, 2); */
    /*affiche_mosaique(vscreen_effect);                 */
}

/*
 * make_effect: fait apparaitre le menu en "eloignant" l'image de jeu
 */

/*void make_effect(int nb_images)
{
    int             i, x1, y1;
    unsigned char   *vscreen_menu;

    vscreen_menu = (unsigned char *)ftris_malloc(scr_nbx*scr_nby, "graphical effect");
    copy_s2v(vscreen_menu);
    copy_s2v(vscr[1]);

    dessine_ecran_bidon(vscreen_menu);

    copy_v2s(vscr[1]);
    //usleep(500000);

    //for (i = 0; i < 20; i++)
    //{
    //    memcpy(vscr[i], vscreen, 64000);
    //    x1 = (96*i)/20;
    //    y1 = (60*i)/20;
    //    nbx = scr_nbx-(x1*2);
    //    for (j = y1; j < 199-y1; j++)
    //        memset(vscr[i]+j*scr_nbx+x1, 1, nbx);
    //}

    for (i = 0; i < nb_images; i++)
    {
        //memcpy(vscr[i], vscreen, 64000);
        memcpy_rapide(vscr[i], vscreen_menu, 16000);
        x1 = (96*(i+1))/nb_images;
        y1 = (60*(i+1))/nb_images;
        zoom(vscreen, scr_nbx, scr_nby, vscr[i], scr_nbx-(x1*2), scr_nby-(y1*2), x1, y1);
    }

    for (i = nb_images-1; i >= 0; i--)
    {
        waitretrace();
        copy_v2s(vscr[i]);
    }
    waitretrace();
    copy_v2s(vscreen);
    usleep(1000000);
    for (i = 0; i < nb_images; i++)
    {
        waitretrace();
        copy_v2s(vscr[i]);
    }

    putsprite_v(vscr[nb_images-1], &tube[couleur_tubes][4], 87, 51);
    putsprite_v(vscr[nb_images-1], &tube[couleur_tubes][5], 223, 51);
    putsprite_v(vscr[nb_images-1], &tube[couleur_tubes][3], 87, 140);
    putsprite_v(vscr[nb_images-1], &tube[couleur_tubes][2], 223, 140);

    waitretrace();
    copy_v2s(vscr[nb_images-1]);
    free(vscreen_menu);
}*/

/*
 * make_effect2: fait apparaitre le menu en "eloignant" l'image de jeu et
 *               en faisant apparaitre le menu des bords
 */

/*void make_effect2(int nb_images)
{
    int             i, j, x1, y1;
    unsigned char   *vscreen_menu;

    vscreen_menu = (unsigned char *)ftris_malloc(scr_nbx*scr_nby, "graphical effect");
    copy_s2v(vscreen_menu);
    copy_s2v(vscr[1]);

    dessine_ecran_bidon(vscreen_menu);

    copy_v2s(vscr[1]);
    //usleep(500000);

    for (i = 0; i < nb_images; i++)
    {
        //memset_rapide(vscr[i], 0, 16000);
        memset(vscr[i], 0, 64000);
        x1 = (96*(i+1))/nb_images;
        y1 = (60*(i+1))/nb_images;
        zoom(vscreen, scr_nbx, scr_nby, vscr[i], scr_nbx-(x1*2), scr_nby-(y1*2), x1, y1);
        for (j = 0; j < 140; j++)
        {
            memcpy(vscr[i]+j*scr_nbx, vscreen_menu+j*scr_nbx+96-x1, x1);
            memcpy(vscr[i]+(j+60)*scr_nbx+224+96-x1, vscreen_menu+(j+60)*scr_nbx+224, x1);
        }
        for (j = 0; j < y1; j++)
        {
            memcpy(vscr[i]+96+j*scr_nbx, vscreen_menu+96+(60-y1+j)*scr_nbx, 224);
            memcpy(vscr[i]+199*scr_nbx-j*scr_nbx, vscreen_menu+140*scr_nbx+(y1-j-1)*scr_nbx, 224);
        }
    }

    for (i = nb_images-1; i >= 0; i--)
    {
        waitretrace();
        copy_v2s(vscr[i]);
    }
    waitretrace();
    copy_v2s(vscreen);
    usleep(1000000);
    for (i = 0; i < nb_images; i++)
    {
        waitretrace();
        copy_v2s(vscr[i]);
    }

    putsprite_v(vscr[nb_images-1], &tube[couleur_tubes][4], 87, 51);
    putsprite_v(vscr[nb_images-1], &tube[couleur_tubes][5], 223, 51);
    putsprite_v(vscr[nb_images-1], &tube[couleur_tubes][3], 87, 140);
    putsprite_v(vscr[nb_images-1], &tube[couleur_tubes][2], 223, 140);

    waitretrace();
    copy_v2s(vscr[nb_images-1]);
    free(vscreen_menu);
}*/

/*
 * make_effect3: fait apparaitre le menu en "eloignant" l'image de jeu et
 *               en faisant apparaitre le menu des bords (du centre vers l'exterieur)
 */

/*void make_effect3(int nb_images)
{
    int             i, j, x1, y1;
    unsigned char   *vscreen_menu;

    vscreen_menu = (unsigned char *)ftris_malloc(scr_nbx*scr_nby, "graphical effect");
    copy_s2v(vscreen_menu);
    copy_s2v(vscr[1]);

    dessine_ecran_bidon(vscreen_menu);

    copy_v2s(vscr[1]);
    //usleep(500000);

    for (i = 0; i < nb_images; i++)
    {
        //memset_rapide(vscr[i], 0, 16000);
        memset(vscr[i], 0, 64000);
        x1 = (96*(i+1))/nb_images;
        y1 = (60*(i+1))/nb_images;
        for (j = 0; j < 140; j++)
        {
            memcpy(vscr[i]+j*scr_nbx+96-x1, vscreen_menu+j*scr_nbx, x1);
            memcpy(vscr[i]+(j+60)*scr_nbx+224, vscreen_menu+(j+60)*scr_nbx+224+96-x1, x1);
        }
        for (j = 0; j < y1; j++)
        {
            memcpy(vscr[i]+96+(60-y1+j)*scr_nbx, vscreen_menu+96+j*scr_nbx, 224);
            memcpy(vscr[i]+140*scr_nbx+(y1-j-1)*scr_nbx, vscreen_menu+199*scr_nbx-j*scr_nbx, 224);
        }
        zoom(vscreen, scr_nbx, scr_nby, vscr[i], scr_nbx-(x1*2), scr_nby-(y1*2), x1, y1);
    }

    for (i = nb_images-1; i >= 0; i--)
    {
        waitretrace();
        copy_v2s(vscr[i]);
    }
    waitretrace();
    copy_v2s(vscreen);
    usleep(1000000);
    for (i = 0; i < nb_images; i++)
    {
        waitretrace();
        copy_v2s(vscr[i]);
    }

    putsprite_v(vscr[nb_images-1], &tube[couleur_tubes][4], 87, 51);
    putsprite_v(vscr[nb_images-1], &tube[couleur_tubes][5], 223, 51);
    putsprite_v(vscr[nb_images-1], &tube[couleur_tubes][3], 87, 140);
    putsprite_v(vscr[nb_images-1], &tube[couleur_tubes][2], 223, 140);

    waitretrace();
    copy_v2s(vscr[nb_images-1]);
    free(vscreen_menu);
}*/

/*
 * affiche_progressif: affiche un ecran progressivement
 */

void affiche_progressif(int no_vscr, int x1, int y1, int x2, int y2)
{
    int     i, j, k, ofs_debut, depasse, largeur, ofs, x;

    ofs_debut = x1+y1*scr_nbx;
    depasse = (y2-y1+1)/4;
    largeur = x2-x1+1;
    for (i = y1-depasse; i <= y2; i++)
    {
        if (i >= y1)
            copy_v2s_xy(no_vscr, x1, i, largeur, 1);
        for (j = 0; j < depasse; j++)
            if ( (i+j >= y1) && (i+j <= y2) )
            {
                ofs = x1+(i+j)*scr_nbx;
                for (k = depasse-j; k > 0; k--)
                {
                    x = rand()%largeur;
                    putpixel_ofs(ECRAN_REEL, ofs+x, getpixel_ofs(no_vscr, ofs+x));
                }
            }
    }
}

/*
 * affiche_spirale: affiche un ecran avec un effet de spirale
 *                  depart: 0=haut gauche  1=haut droite  2=bas droite  3=bas gauche
 *                  sens:  1 = sens des auguille d'une montre
 *                        -1 = sens inverse des aiguilles d'une montre
 *                  vitesse: 0 = mini
 */

void affiche_spirale(int no_vscr, int largeur, int hauteur,
                     int depart, int sens, int vitesse)
{
    int     i, x, y, direc, nb_cases;
    int     x1, y1, x2, y2;
    int     x_dir_inc[4] = {  0, 1, 0, -1 };
    int     y_dir_inc[4] = { -1, 0, 1,  0 };

    /*switch (depart)
    {
        case 0:
            x = 0;
            y = 0;
            direc = 1;
            break;
        case 1:
            x = 0;
            y = 0;
            direc = 1;
            sens = 1;
            break;
    }*/
    x = 0;
    y = 0;
    direc = 1;
    sens = 1;
    nb_cases = (scr_nbx*scr_nby)/(largeur*hauteur);
    x1 = 0; x2 = scr_nbx-largeur;
    y1 = 0; y2 = scr_nby-hauteur;
    for (i = 0; i < nb_cases; i++)
    {
        if (i%vitesse == 0)
            waitretrace();
        copy_v2s_xy(no_vscr, x, y, largeur, hauteur);

        if (x+(x_dir_inc[direc]*largeur) > x2)
        {
            direc = (direc+sens)%4;
            x2 = x;
            y1 += (hauteur*sens);
        }
        else
            if (x+(x_dir_inc[direc]*largeur) < x1)
            {
                direc = (direc+sens)%4;
                x1 = x;
                y2 -= (hauteur*sens);
            }
            else
                if (y+(y_dir_inc[direc]*hauteur) > y2)
                {
                    direc = (direc+sens)%4;
                    y2 = y;
                    x2 -= (hauteur*sens);
                }
                else
                    if (y+(y_dir_inc[direc]*hauteur) < y1)
                    {
                        direc += (direc+sens)%4;
                        y1 = y;
                        x1 += (hauteur*sens);
                    }

        x += x_dir_inc[direc]*largeur;
        y += y_dir_inc[direc]*hauteur;
    }
}

void affiche_mosaique(int no_vscr)
{
    int     i, x, y;
    int     mosaique[640] = { 629,164,581,378,497,325,342,193,263,168,
                              301,625,27,406,117,345,103,632,494,636,
                              276,253,264,208,315,281,243,207,428,489,
                              140,417,13,594,28,383,151,370,576,414,
                              410,109,271,437,515,388,15,619,252,509,
                              615,528,634,112,608,181,265,212,389,565,
                              61,401,343,587,355,202,506,485,138,256,
                              248,424,635,45,580,486,297,321,334,58,
                              188,318,26,369,583,110,118,44,392,84,
                              631,107,327,65,614,172,203,127,323,423,
                              354,191,468,166,37,126,360,56,548,561,
                              277,505,268,246,20,330,612,529,613,240,
                              362,436,396,588,627,224,626,536,222,346,
                              150,254,105,532,566,170,32,279,52,251,
                              51,135,160,537,69,400,431,68,137,55,
                              237,592,456,584,92,607,198,25,72,602,
                              464,242,508,393,558,115,516,482,250,36,
                              379,319,308,120,622,377,258,233,304,492,
                              471,128,287,275,219,320,582,333,186,322,
                              86,560,510,111,179,94,448,199,300,542,
                              93,481,189,185,628,547,175,227,413,312,
                              500,390,621,209,541,77,407,598,556,278,
                              178,520,451,593,292,142,31,280,206,148,
                              134,434,380,184,361,200,266,149,545,405,
                              503,639,14,49,74,465,2,47,17,78,
                              269,578,157,630,599,225,477,502,449,574,
                              75,205,356,96,555,146,195,454,616,16,
                              444,575,241,174,9,108,597,226,64,450,
                              426,341,409,604,586,238,496,97,475,472,
                              113,591,479,176,637,197,617,317,42,106,
                              340,526,553,452,411,395,546,535,461,71,
                              216,499,130,498,158,470,24,255,512,595,
                              35,511,303,459,324,145,326,57,139,519,
                              284,466,399,549,294,420,463,147,21,585,
                              433,79,572,507,347,19,337,440,220,307,
                              443,404,551,99,311,335,0,563,603,89,
                              221,328,161,229,386,143,236,491,11,620,
                              415,162,467,231,85,70,478,125,539,554,
                              272,286,298,114,568,609,285,53,531,533,
                              213,43,427,416,190,571,349,274,288,121,
                              501,633,141,29,59,339,488,358,210,1,
                              458,101,579,606,372,570,348,344,538,48,
                              244,183,211,514,453,201,102,129,544,290,
                              313,429,418,527,173,601,63,309,245,54,
                              567,447,261,73,62,152,425,368,136,474,
                              40,192,91,543,387,187,8,336,306,155,
                              119,391,366,87,524,385,530,249,81,397,
                              217,34,159,88,490,412,262,398,455,260,
                              600,518,457,552,350,310,122,132,522,228,
                              33,12,39,5,364,30,60,605,611,123,
                              523,98,167,259,589,156,214,329,257,352,
                              314,484,104,305,351,473,235,293,381,480,
                              462,439,521,610,403,270,483,296,144,596,
                              402,18,577,267,338,182,90,124,353,289,
                              562,196,534,283,357,131,371,10,232,557,
                              331,376,359,476,46,95,67,154,239,282,
                              441,495,564,419,76,80,517,234,363,525,
                              223,165,590,573,446,169,7,4,365,430,
                              299,623,204,421,374,422,460,302,230,435,
                              438,218,432,133,442,41,215,550,569,394,
                              504,540,38,273,559,171,22,3,382,177,
                              6,469,332,316,445,83,295,163,367,116,
                              180,493,487,100,513,194,291,66,375,50,
                              618,384,408,638,624,82,247,373,153,23 };

    for (i = 0; i < 640; i++)
    {
        if (i%10 == 0)
            waitretrace();
        x = (mosaique[i]%32)*10;
        y = (mosaique[i]/32)*10;
        copy_v2s_xy(VSCR_DERNIER, x, y, 10, 10);
    }
}
