/* ====
   ==== FlashTris Game, (c) 1999-2001 by FlashCode (Sebastien Helleu)
   ====
   ====    web: http://flashcode.free.fr
   ==== e-mail: flashcode@free.fr
   ====
   ==== menu_gfx.c: routines graphiques pour le menu de FlashTris
   ====

   ==== This file is part of FlashTris.
   ====
   ==== FlashTris is free software; you can redistribute it and/or modify
   ==== it under the terms of the GNU General Public License as published by
   ==== the Free Software Foundation; either version 2 of the License, or
   ==== (at your option) any later version.
   ====
   ==== FlashTris is distributed in the hope that it will be useful,
   ==== but WITHOUT ANY WARRANTY; without even the implied warranty of
   ==== MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   ==== GNU General Public License for more details.
   ====
   ==== You should have received a copy of the GNU General Public License
   ==== along with FlashTris; if not, write to the Free Software
   ==== Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Auteur  : FlashCode <flashcode@free.fr>
Creation: 1999/06/21

2001/09/06: FlashCode:
    - ajout de la licence
*/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifndef MSDOS
    #include <vgakeyboard.h>
#endif

#include "ftris.h"
#include "menu_gfx.h"
#include "menu.h"
#include "menu_tst.h"
#include "game.h"
#include "key.h"
#include "netftris.h"
#include "about.h"
#include "effet.h"
#include "video.h"
#include "./module/mod.h"


/*
 * save_graphics_raw: sauve l'image courante en RAW
 */

void save_graphics_raw()
{
    FILE            *f;

    f = fopen("gfx.raw", "wb");
    /*fwrite(pal_ftris, 768, 1, f);*/
    fwrite(palette, 768, 1, f);
    copy_s2v(VSCR_DERNIER);
    fwrite(vscreen[VSCR_DERNIER], 64000, 1, f);
    fclose(f);
}

/*
 * affiche_tubes_menu: affiche les tubes du menu selon la couleur desiree
 */

void affiche_tubes_menu(int no_vscr, int couleur)
{
    int     i, advanced_menu_general;

    /*  Ordre de tracage des barres:
           1
           1222222222
           1      4
        33333333334
                  4   */

    advanced_menu_general = (pos_menu == 0) && (pos_advanced == 1);

    /* 1 */
    for (i = 0; i < 16; i++)
        putsprite(no_vscr, &tube[couleur][0], 87, i*9);
    if (!advanced_menu_general)
        putsprite(no_vscr, &tube[couleur][4], 87, 51);
    /* 2 */
    if (advanced_menu_general)
    {
        for (i = 15; i < 24; i++)
            putsprite(no_vscr, &tube[couleur][1], 96+i*9, 51);
        putsprite(no_vscr, &tube[tab_couleur_tubes[couleur_tubes]][6], 224, 51);
    }
    else
    {
        for (i = 0; i < 24; i++)
            putsprite(no_vscr, &tube[couleur][1], 96+i*9, 51);
        putsprite(no_vscr, &tube[couleur][5], 224, 51);
    }
    putsprite_clip(no_vscr, &tube[couleur][1], 312, 51, 0, 0, 8, 9);
    /* 3 */
    for (i = 0; i < 25; i++)
        putsprite(no_vscr, &tube[couleur][1], i*9, 140);
    putsprite(no_vscr, &tube[couleur][3], 87, 140);
    /* 4 */
    for (i = 0; i < 15; i++)
        putsprite(no_vscr, &tube[couleur][0], 224, 60+i*9);
    putsprite_clip(no_vscr, &tube[couleur][0], 224, 195, 0, 0, 9, 5);
    putsprite(no_vscr, &tube[couleur][2], 224, 140);
}

/*
 * get_ptr_libelle: renvoie le libelle d'une valeur pour une option
 *                  en fonction de la langue
 *                  numero_libelle est le numero du libelle recherche
 *                    s'l vaut -1, on prend le libelle du choix selectionne
 */

unsigned char *get_ptr_libelle(t_option *option, int numero_libelle)
{
    unsigned char   *ptr_libelle;

    if (numero_libelle == -1)
        numero_libelle = (*(option->o_valeur_int))-(option->o_mini);
    ptr_libelle = option->o_ptr_libelle[numero_libelle];
    if (langue == ANGLAIS)
        return ptr_libelle;
    else
        return ptr_libelle+strlen(ptr_libelle)+1;
}

/*
 * dessin_cadre_option: dessine le cadre autour d'une option (bouton ou texte)
 */

void dessin_cadre_option(int x, int y, int largeur_cadre, int hauteur_cadre)
{
    int     i;

    /* haut a gauche et bas a gauche */
    for (i = x-1; i <= x+largeur_cadre/2; i++)
    {
        putpixel_ofs(ECRAN_REEL, i+(y-2)*scr_nbx, 160+((i-x+1)*14)/(largeur_cadre/2+1));
        putpixel_ofs(ECRAN_REEL, i+(y+hauteur_cadre+2)*scr_nbx, 174-((i-x+1)*14)/(largeur_cadre/2+1));
    }
    /* haut a droite et bas a droite */
    for (i = x+largeur_cadre/2+1; i <= x+largeur_cadre+1; i++)
    {
        putpixel_ofs(ECRAN_REEL, i+(y-2)*scr_nbx, 174-((i-x-largeur_cadre/2-1)*14)/(largeur_cadre/2+1));
        putpixel_ofs(ECRAN_REEL, i+(y+hauteur_cadre+2)*scr_nbx, 160+((i-x-largeur_cadre/2-1)*14)/(largeur_cadre/2+1));
    }
    /* gauche et droite */
    for (i = y-1; i <= y+hauteur_cadre+1; i++)
    {
        /*putpixel_ofs(ECRAN_REEL, x-2+i*scr_nbx, 170-((i-y+1)*10)/8);*/
        /*putpixel_ofs(ECRAN_REEL, x+largeur_cadre+2+i*scr_nbx, 170-((i-y+1)*10)/8);*/
        putpixel_ofs(ECRAN_REEL, x-2+i*scr_nbx, 170-((i-y+1)*10)/(hauteur_cadre+3));
        putpixel_ofs(ECRAN_REEL, x+largeur_cadre+2+i*scr_nbx, 170-((i-y+1)*10)/(hauteur_cadre+3));
    }
}

/*
 * affiche_option: affiche une option (peut etre un bouton)
 *                 Renvoie la hauteur et la largeur dessinnees
 *                   sous la forme (largeur << 8)+hauteur
 *                 (la largeur n'est alimentee que s'il s'agit d'un bouton)
 *                 redessine_fond: 0 = rien   1 = valeur   2 = tout
 */

int affiche_option(int no_vscr, int no_menu, int advanced, int no_option,
                   int menu_actif, int opt_active, int redessine_fond)
{
    unsigned char   *ptr_nom, libelle[50], chaine_tmp[1000], chaine_tmp2[1000];
    int             coul_nom, coul_ombre_nom, coul_libelle, coul_ombre_libelle;
    int             i, j, x, y, largeur_opt, largeur_nom, largeur_cadre;
    int             y_inc, bouton;
    t_option        *option;

    option = &ptr_menus[no_menu][advanced][no_option];
    if (option->o_choix_actif)
    {
        if (menu_actif)
        {
            if (opt_active)
            {
                coul_nom = COUL_OPTMENU_ACT_NOM;
                coul_ombre_nom = COUL_OPTMENU_ACT_OMBRE_NOM;
                coul_libelle = COUL_OPTMENU_ACT_LIBELLE;
                coul_ombre_libelle = COUL_OPTMENU_ACT_OMBRE_LIB;
            }
            else
            {
                coul_nom = COUL_MENU_ACT_NOM;
                coul_ombre_nom = COUL_MENU_ACT_OMBRE_NOM;
                coul_libelle = COUL_MENU_ACT_LIBELLE;
                coul_ombre_libelle = COUL_MENU_ACT_OMBRE_LIB;
            }
        }
        else
        {
            coul_nom = COUL_OPT_INACT_NOM;
            coul_ombre_nom = COUL_OPT_INACT_OMBRE_NOM;
            coul_libelle = COUL_OPT_INACT_LIBELLE;
            coul_ombre_libelle = COUL_OPT_INACT_OMBRE_LIB;
        }
    }
    else
    {
        coul_nom = COUL_OPT_GRISEE_NOM;
        coul_ombre_nom = COUL_OPT_GRISEE_OMBRE_NOM;
        coul_libelle = COUL_OPT_GRISEE_LIBELLE;
        coul_ombre_libelle = COUL_OPT_GRISEE_OMBRE_LIB;
    }

    x = pos_option[no_menu][advanced][no_option][0];
    y = pos_option[no_menu][advanced][no_option][1];
    largeur_opt = largeur_option[no_menu][advanced][no_option];
    if (langue == ANGLAIS)
        ptr_nom = option->o_nom;
    else
        ptr_nom = option->o_nom+strlen(option->o_nom)+1;
    /* Rajout de lignes */
    y_inc = 0;
    while ( (ptr_nom[0] >= '0') && (ptr_nom[0] <= '9') )
    {
        y_inc += (ptr_nom[0]-'0');
        ptr_nom++;
    }
    y = y+y_inc;
    /* Bouton ? */
    if (ptr_nom[0] == '*')
    {
        ptr_nom++;
        bouton = 1;
    }
    else
        bouton = 0;
    largeur_nom = text_length(ptr_nom)+bouton;
    if (largeur_opt == 0)
    {
        largeur_opt = largeur_nom;
        largeur_option[no_menu][advanced][no_option] = largeur_opt;
    }
    if (bouton)
        largeur_option_reelle[no_menu][advanced][no_option] = largeur_nom;
    else
        largeur_option_reelle[no_menu][advanced][no_option] =
            largeur_option[no_menu][advanced][no_option];

    /* redessine le fond de l'option */
    if (redessine_fond == 2)
    {
        if (no_vscr == ECRAN_REEL)
            copy_v2s_xy(VSCR_MENU, x-2, y-2, largeur_opt+5, 10);
        else
            copy_v2v_xy(VSCR_MENU, no_vscr, x-2, y-2, largeur_opt+5, 10);
    }
    else
        if (redessine_fond == 1)
        {
            if (no_vscr == ECRAN_REEL)
                copy_v2s_xy(VSCR_MENU, x+largeur_nom+1, y,
                    largeur_opt-(largeur_nom+1)+1, 6);
            else
                copy_v2v_xy(VSCR_MENU, no_vscr, x+largeur_nom+1, y,
                    largeur_opt-(largeur_nom+1)+1, 6);
        }

    /* dessin du cadre violet autour de l'option et assombrissement du fond */
    if ( (menu_actif) && (opt_active) )
    {
        largeur_cadre = (bouton)?largeur_nom:largeur_opt;
        dessin_cadre_option(x, y, largeur_cadre, 5);
        for (i = x-1; i < x+largeur_cadre+2; i++)
            for (j = y-1; j <= y+6; j++)
                assombrit_pixel(VSCR_MENU, ECRAN_REEL, i, j, 159, 159, 5);
    }

    if (bouton)
    {
        /*if ( (menu_actif) && (opt_active) )
            affiche_bouton(x, y, coul_nom, -1, 124, //coul_fond_bouton[0],
                0, 0, 1, ptr_nom);
        else
            affiche_bouton(x, y, coul_nom, -1, 26, //coul_fond_bouton[0],
                0, 0, 1, ptr_nom);*/
        for (i = x+1; i < x+largeur_nom; i++)
        {
            assombrit_pixel(VSCR_MENU, ECRAN_REEL, i, y-1, 110, 110, 5);
            assombrit_pixel(VSCR_MENU, ECRAN_REEL, i, y+6, 110, 110, 5);
        }
        for (i = x; i <= x+largeur_nom; i++)
        {
            assombrit_pixel(VSCR_MENU, ECRAN_REEL, i, y, 110, 110, 5);
            assombrit_pixel(VSCR_MENU, ECRAN_REEL, i, y+5, 110, 110, 5);
        }
        for (j = y+1; j < y+5; j++)
            for (i = x-1; i <= x+largeur_nom+1; i++)
                assombrit_pixel(VSCR_MENU, ECRAN_REEL, i, j, 110, 110, 5);
        write_xy_s_c_o(no_vscr, x+1, y, coul_nom, -1, coul_ombre_nom, ptr_nom);
        return ((largeur_nom+4) << 8)+5+y_inc;
    }
    else
    {
        if (option->o_valeur_int != NULL)   /* entier */
        {
            /* case a cocher ? (==> valeur possible 0 ou 1 et pas de libelles) */
            if ( (option->o_mini == 0) && (option->o_maxi == 1)
                && (option->o_ptr_libelle == NULL) )
            {
                if (*(option->o_valeur_int) == 0)
                {
                    if (option->o_choix_actif)
                        putsprite_tr(&case_vide[0], x+largeur_opt-6, y);
                    else
                        putsprite_tr(&case_vide[1], x+largeur_opt-6, y);
                }
                else
                {
                    if (option->o_choix_actif)
                        putsprite_tr(&case_cochee[couleur_cases_cochees], x+largeur_opt-6, y);
                    else
                        putsprite_tr(&case_cochee[5], x+largeur_opt-6, y);
                }
            }
            else
            {
                /* libelle a prendre dans un tableau de chaines ? */
                if (option->o_ptr_libelle != NULL)
                    strcpy(libelle, get_ptr_libelle(option, -1));
                else
                    // valeur brute
                    sprintf(libelle, "%d", *(option->o_valeur_int));
                /*x = x+(42-text_length(ptr_nom));*/
                /*write_xy_s_c(ECRAN_REEL, x, y, 70, 1, ptr_nom);*/
                write_xy_s_c_o_brut(no_vscr, x+largeur_opt-text_length(libelle), y,
                    coul_libelle, -1, coul_ombre_libelle, libelle);
            }
            write_xy_s_c_o(no_vscr, x, y, coul_nom, -1, coul_ombre_nom, ptr_nom);
            return (largeur_opt << 8)+5+y_inc;
        }
        else
            if (option->o_valeur_chaine != NULL)
            {
                /* chaine */
                j = largeur_opt-text_length(ptr_nom)-4;
                if (text_length(option->o_valeur_chaine) > j)
                {
                    strcpy(chaine_tmp, option->o_valeur_chaine);
                    i = strlen(chaine_tmp);
                    do
                    {
                        chaine_tmp[i] = '\0';
                        strcpy(chaine_tmp2, chaine_tmp);
                        strcat(chaine_tmp2, "...");
                        i--;
                    } while ( (text_length(chaine_tmp2) > j) && (i > 0) );
                    strcat(chaine_tmp, "...");
                }
                else
                    strcpy(chaine_tmp, option->o_valeur_chaine);

                write_xy_s_c_o(no_vscr, x, y, coul_nom, -1, coul_ombre_nom, ptr_nom);
                write_xy_s_c_o(no_vscr, x+largeur_opt-text_length(chaine_tmp),
                    y, coul_libelle, -1, coul_ombre_libelle, chaine_tmp);
                return (largeur_opt << 8)+5+y_inc;
            }
            else
            {
                /* texte simple */
                write_xy_s_c_o(no_vscr, x, y, coul_nom, -1, coul_ombre_nom, ptr_nom);
                return (largeur_opt << 8)+5+y_inc;
            }
    }
    /* jamais atteint */
    return 0;
}

/*
 * liste_choix_affiche_option: affiche une option de la liste de choix
 */

void liste_choix_affiche_option(int pos_option)
{
    int     i, j, coul_fond, facteur;

    if (pos_option == liste_choix_pos)
    {
        /* fond bleu plus clair pour l'option active */
        /*coul_fond = 150;
        facteur = 6;*/
        coul_fond = 41;
        facteur = 8;
    }
    else
    {
        coul_fond = 159;    /* fond bleu pour une option inactive */
        facteur = 5;
    }
    for (i = liste_choix_x1+1; i < liste_choix_x2; i++)
        for (j = liste_choix_y[pos_option]-2; j <= liste_choix_y[pos_option]+5+1; j++)
            assombrit_pixel(VSCR_AUTRES+3, ECRAN_REEL, i, j, coul_fond, coul_fond, facteur);

    if (liste_choix_actif[pos_option])
    {
        if (pos_option == liste_choix_pos)
            write_xy_s_c_o_brut(ECRAN_REEL, liste_choix_x[pos_option], liste_choix_y[pos_option],
                COUL_OPTLISTE_ACT_LIB, -1, COUL_OPTLISTE_ACT_OMBRE,
                liste_choix_lib[pos_option]);
        else
            write_xy_s_c_o_brut(ECRAN_REEL, liste_choix_x[pos_option], liste_choix_y[pos_option],
                COUL_OPTLISTE_INACT_LIB, -1, COUL_OPTLISTE_INACT_OMBRE,
                liste_choix_lib[pos_option]);
    }
    else
        write_xy_s_c_o_brut(ECRAN_REEL, liste_choix_x[pos_option],
            liste_choix_y[pos_option], 27, -1, 29, liste_choix_lib[pos_option]);
}

/*
 * affiche_liste_choix: affiche une liste de choix pour l'option selectionnee
 *                      Attention: l'option selectionnee ne doit etre ni un
 *                                 bouton ni une case a cocher ni une chaine
 *                                 a saisir
 */

void affiche_liste_choix()
{
    #define         HAUTEUR_CHOIX   9

    int             i, largeur_maxi, largeur;
    int             x1, x2, y1, y2;
    t_option        *option;
    unsigned char   *ptr_libelle;
    unsigned char   libelle[50];

    if ( (sb16) && (sons_menu) )
        SON_OUVRE_LISTE_CHOIX;

    option = &ptr_menus[pos_menu][pos_advanced][pos_barre];

    /* calcule le nombre de choix */
    liste_choix_nb = option->o_maxi-option->o_mini+1;

    /* choix courant = valeur de l'option */
    liste_choix_pos = *(option->o_valeur_int)-option->o_mini;

    /* stockage des noms de choix */
    if (option->o_ptr_libelle == NULL)
    {
        /* entiers a convertir en chaine */
        for (i = 0; i < liste_choix_nb; i++)
        {
            sprintf(libelle, "%d", option->o_mini+i);
            liste_choix_lib[i] = (unsigned char *)ftris_malloc(strlen(libelle)+1, "choice list");
            strcpy(liste_choix_lib[i], libelle);
        }
    }
    else
    {
        /* libelles a recuperer dans le tableau des libelles en fonction de la langue */
        for (i = 0; i < liste_choix_nb; i++)
        {
            ptr_libelle = get_ptr_libelle(option, i);
            liste_choix_lib[i] = (unsigned char *)ftris_malloc(strlen(ptr_libelle)+1, "choice list");
            strcpy(liste_choix_lib[i], ptr_libelle);
        }
    }

    /* recherche la longueur maxi (en pixels) d'un choix */
    largeur_maxi = 0;
    for (i = 0; i < liste_choix_nb; i++)
    {
        largeur = text_length(liste_choix_lib[i]);
        liste_choix_x[i] = largeur;
        if (largeur > largeur_maxi)
            largeur_maxi = largeur;
    }
    if (largeur_maxi < 20)
        largeur_maxi = 20;

    /* calcule la position de l'aire pour la liste de choix (x1,y1) --> (x2,y2) */
    liste_choix_x2 = pos_option[pos_menu][pos_advanced][pos_barre][0]+
        largeur_option[pos_menu][pos_advanced][pos_barre]+2;
    liste_choix_x1 = liste_choix_x2-largeur_maxi-4;
    liste_choix_y1 = pos_option[pos_menu][pos_advanced][pos_barre][1]-
        (liste_choix_nb/2)*HAUTEUR_CHOIX;
    if (liste_choix_y1 < 0)
        liste_choix_y1 = 0;
    liste_choix_y2 = liste_choix_y1+liste_choix_nb*HAUTEUR_CHOIX+1;
    if (liste_choix_y2 > 199)
    {
        liste_choix_y1 -= (liste_choix_y2-199);
        liste_choix_y2 = 199;
    }

    /* calcule la position de chaque option dans l'aire calculee ci-dessus */
    /* et verifie si le choix est "actif" ( = non grise)                   */
    for (i = 0; i < liste_choix_nb; i++)
    {
        /* cale a droite... : */
        liste_choix_x[i] = liste_choix_x2-liste_choix_x[i]-2;
        /* ... ou centre : */
        /*liste_choix_x[i] = liste_choix_x1+(largeur_maxi-liste_choix_x[i])/2+2;*/
        liste_choix_y[i] = liste_choix_y1+2+i*HAUTEUR_CHOIX+1;

        /* verifie si le choix est "actif" */
        liste_choix_actif[i] =
            (option->o_choix_actif)
            && ( (option->o_fonction_verifie == NULL)
                 || (verifie_option(option->o_fonction_verifie, option->o_mini+i)));
    }

    /* sauvegarde le fond de l'image */
    getsprite(ECRAN_REEL, liste_choix_x1, liste_choix_y1, liste_choix_x2-liste_choix_x1+1,
        liste_choix_y2-liste_choix_y1+1, &liste_choix_fond);

    /* place cette image dans VSCR_AUTRES+3 (pour l'assombrissement) */
    putsprite(VSCR_AUTRES+3, &liste_choix_fond, liste_choix_x1, liste_choix_y1);

    /* replace le fond de l'image sur l'emplacement du libelle */
    copy_v2v_xy(VSCR_AUTRES, VSCR_AUTRES+3,
        liste_choix_x1, pos_option[pos_menu][pos_advanced][pos_barre][1]-2,
        liste_choix_x2-liste_choix_x1+1, 10);
    if (liste_choix_y1 != pos_option[pos_menu][pos_advanced][pos_barre][1]-2)
        assombrit_pixel(VSCR_AUTRES+3, ECRAN_REEL, liste_choix_x1,
            pos_option[pos_menu][pos_advanced][pos_barre][1]-2,
            159, 159, 5);
    if (liste_choix_y2 != pos_option[pos_menu][pos_advanced][pos_barre][1]+7)
        assombrit_pixel(VSCR_AUTRES+3, ECRAN_REEL, liste_choix_x1,
            pos_option[pos_menu][pos_advanced][pos_barre][1]+7,
            159, 159, 5);

    /* affichage des options (ne redessine le fond que pour l'option active) */
    for (i = 0; i < liste_choix_nb; i++)
        liste_choix_affiche_option(i);

    /* affichage du cadre violet :            8
                                          /-------\
                                          |       |
                                          |       |
                                         9|       | 7
                                          |       |
                                          |       |
                                    1     |       |
                               /----------/       |
                             2 |                  |     -  -  -  -  -
                               \----------\       |
                                    3     |       |
                                          |       |
                                         4|       | 6
                                          |       |
                                          |       |
                                          \-------/
                                              5              */
    /* 1 */
    x1 = pos_option[pos_menu][pos_advanced][pos_barre][0]-1;
    x2 = liste_choix_x1-1;
    y1 = pos_option[pos_menu][pos_advanced][pos_barre][1]-2;
    for (i = x1; i <= x2; i++)
        putpixel_ofs(ECRAN_REEL, i+y1*scr_nbx, 160+((i-x1+1)*14)/(x2-x1+1));
    /* 2 */
    y1 = pos_option[pos_menu][pos_advanced][pos_barre][1]-1;
    y2 = y1+7;
    x1 = pos_option[pos_menu][pos_advanced][pos_barre][0]-2;
    for (i = y1; i <= y2; i++)
        putpixel_ofs(ECRAN_REEL, x1+i*scr_nbx, 170-((i-y1+1)*10)/8);
    /* 3 */
    x1 = pos_option[pos_menu][pos_advanced][pos_barre][0]-1;
    if (liste_choix_y2 == pos_option[pos_menu][pos_advanced][pos_barre][1]+7)
        x2 = liste_choix_x2-1;
    else
        x2 = liste_choix_x1-1;
    y1 = pos_option[pos_menu][pos_advanced][pos_barre][1]+7;
    for (i = x1; i <= x2; i++)
        putpixel_ofs(ECRAN_REEL, i+y1*scr_nbx, 174-((i-x1+1)*14)/(x2-x1+1));
    if (liste_choix_y2 != pos_option[pos_menu][pos_advanced][pos_barre][1]+7)
    {
    /* 4 */
        y1 = pos_option[pos_menu][pos_advanced][pos_barre][1]+8;
        y2 = liste_choix_y2-1;
        x1 = liste_choix_x1;
        for (i = y1; i <= y2; i++)
            putpixel_ofs(ECRAN_REEL, x1+i*scr_nbx, 174-((i-y1)*14)/(y2-y1+1));
    /* 5 */
        x1 = liste_choix_x1+1;
        x2 = liste_choix_x2-1;
        y1 = liste_choix_y2;
        for (i = x1; i <= x2; i++)
            putpixel_ofs(ECRAN_REEL, i+y1*scr_nbx, 174-((i-x1+1)*14)/(x2-x1+1));
    }
    /* 6 */
    y1 = liste_choix_y1+(liste_choix_y2-liste_choix_y1)/2;
    y2 = liste_choix_y2-1;
    x1 = liste_choix_x2;
    for (i = y1; i <= y2; i++)
        putpixel_ofs(ECRAN_REEL, x1+i*scr_nbx, 160+((i-y1)*14)/(y2-y1+1));
    /* 7 */
    y1 = liste_choix_y1+1;
    y2 = liste_choix_y1+(liste_choix_y2-liste_choix_y1)/2-1;
    x1 = liste_choix_x2;
    for (i = y1; i <= y2; i++)
        putpixel_ofs(ECRAN_REEL, x1+i*scr_nbx, 174-((i-y1)*14)/(y2-y1+1));
    /* 8 */
    x1 = liste_choix_x1+1;
    x2 = liste_choix_x2-1;
    y1 = liste_choix_y1;
    for (i = x1; i <= x2; i++)
        putpixel_ofs(ECRAN_REEL, i+y1*scr_nbx, 174-((i-x1+1)*14)/(x2-x1+1));
    /* 9 */
    y1 = liste_choix_y1+1;
    y2 = pos_option[pos_menu][pos_advanced][pos_barre][1]-3;
    x1 = liste_choix_x1;
    for (i = y1; i <= y2; i++)
        putpixel_ofs(ECRAN_REEL, x1+i*scr_nbx, 160+((i-y1)*14)/(y2-y1+1));

    liste_choix = 1;
    liste_choix_cptr = DUREE_LISTE_CHOIX;
}

/*
 * efface_liste_choix: efface la liste de choix affichee pour l'option selectionnee
 */

void efface_liste_choix()
{
    int     i;

    if ( (sb16) && (sons_menu) )
        SON_FERME_LISTE_CHOIX;

    for (i = 0; i < liste_choix_nb; i++)
        free(liste_choix_lib[i]);
    putsprite(ECRAN_REEL, &liste_choix_fond, liste_choix_x1, liste_choix_y1);
    free(liste_choix_fond.s_ptr);
    dessin_cadre_option(pos_option[pos_menu][pos_advanced][pos_barre][0],
        pos_option[pos_menu][pos_advanced][pos_barre][1],
        largeur_option[pos_menu][pos_advanced][pos_barre], 5);

    liste_choix = 0;
}

/*
 * efface_bulle: efface la bulle affichee au dessus/dessous d'une option
 */

void efface_bulle()
{
    if (bulle_affichee)
    {
        pixels_efface();
        pixels_init();
        putsprite(ECRAN_REEL, &bulle_fond, bulle_x1, bulle_y1);
        free(bulle_fond.s_ptr);
        /*copy_v2s_xy(VSCR_MENU, bulle_x1, bulle_y1, bulle_x2-bulle_x1+1,
            bulle_y2-bulle_y1+1);*/
        bulle_affichee = 0;
    }
    bulle_cptr = 0;
}

/*
 * affiche_bulle: affiche une bulle au dessus/dessous d'une option
 *                duree_bulle: duree d'affichage de la bulle en nombre de
 *                             rafraichissements d'ecran
 *                type_bulle : 0 = bulle d'aide sur l'option
 *                             1 = bulle d'info sur la valeur choisie
 */

void affiche_bulle(int type_bulle, int duree_bulle, int no_menu, int advanced,
                   int no_option)
{
    int             i, j;
    t_option        *option;
    unsigned char   *ptr_texte;
    unsigned char   texte_bulle[128];
    int             largeur_bulle;
    int             coul_texte, coul_fond, facteur;

    efface_bulle();
    option = &ptr_menus[no_menu][advanced][no_option];
    ptr_texte = NULL;
    switch (type_bulle)
    {
        case BULLE_AIDE_OPTION:
            if (option->o_bulle != NULL)
            {
                if (langue == ANGLAIS)
                    ptr_texte = option->o_bulle;
                else
                    ptr_texte = option->o_bulle+strlen(option->o_bulle)+1;
            }
            break;
        case BULLE_INFO_CHOIX:
            if (option->o_ptr_bulle != NULL)
            {
                ptr_texte = option->o_ptr_bulle[*(option->o_valeur_int)-option->o_mini];
                if (langue == FRANCAIS)
                    ptr_texte = ptr_texte+strlen(ptr_texte)+1;
            }
            break;
    }
    if (ptr_texte == NULL)
    {
        if (langue == ANGLAIS)
            strcpy(texte_bulle, "No help!");
        else
            strcpy(texte_bulle, "Pas d'aide!");
        coul_texte = 16;
        coul_fond = 41;     /* fond rouge: erreur */
        facteur = 5;
    }
    else
    {
        strcpy(texte_bulle, ptr_texte);
        if (type_bulle == BULLE_AIDE_OPTION)
        {
            coul_texte = 64;
            coul_fond = 63;
            facteur = 7;
        }
        else
        {
            coul_texte = 112;
            coul_fond = 152;
            facteur = 5;
        }
    }
    largeur_bulle = text_length(texte_bulle)+4;

    if (type_bulle == BULLE_AIDE_OPTION)
        bulle_x1 = pos_option[no_menu][advanced][no_option][0]-2;
    else
        bulle_x1 = pos_option[no_menu][advanced][no_option][0]+2+
            largeur_option[no_menu][advanced][no_option]-largeur_bulle;

    if (pos_option[no_menu][advanced][no_option][1]-9 < 0)
        bulle_y1 = pos_option[no_menu][advanced][no_option][1]+8;
    else
        bulle_y1 = pos_option[no_menu][advanced][no_option][1]-11;

    if (bulle_x1 < 0)
        bulle_x1 = 0;
    else
        if (bulle_x1+largeur_bulle-1 > 319)
            bulle_x1 = bulle_x1-(bulle_x1+largeur_bulle-1-319);

    bulle_x2 = bulle_x1+largeur_bulle-1;
    bulle_y2 = bulle_y1+8;

    /* sauve le fond de la bulle */
    getsprite(ECRAN_REEL, bulle_x1, bulle_y1, bulle_x2-bulle_x1+1, bulle_y2-bulle_y1+1,
        &bulle_fond);

    /* place cette image dans VSCR_AUTRES+3 (pour l'assombrissement) */
    putsprite(VSCR_AUTRES+3, &bulle_fond, bulle_x1, bulle_y1);

    /*  asombrit le fond de la bulle */
    for (i = bulle_x1+1; i < bulle_x2; i++)
    {
        assombrit_pixel(VSCR_AUTRES+3, ECRAN_REEL, i, bulle_y1, coul_fond, coul_fond, facteur);
        assombrit_pixel(VSCR_AUTRES+3, ECRAN_REEL, i, bulle_y2, coul_fond, coul_fond, facteur);
    }
    for (i = bulle_x1; i <= bulle_x2; i++)
        for (j = bulle_y1+1; j < bulle_y2; j++)
            assombrit_pixel(VSCR_AUTRES+3, ECRAN_REEL, i, j, coul_fond, coul_fond, facteur);

    /* affiche le texte de la bulle */
    write_xy_s_c(ECRAN_REEL, bulle_x1+2, bulle_y1+2, coul_texte, 1, texte_bulle);

    bulle_duree = duree_bulle;
    if (strlen(texte_bulle) > 30)
        bulle_duree += (strlen(texte_bulle)-30)*4;
    bulle_affichee = 1;
}

/* edition_chaine_affiche: affiche la chaine editee */

void edition_chaine_affiche(int efface_fond)
{
    int     i, largeur_max, longueur_chaine;

    if (efface_fond)
        copy_v2s_xy(VSCR_AUTRES+3, edition_chaine_x1, edition_chaine_y1,
            (edition_chaine_x2-edition_chaine_x1)+1,
            (edition_chaine_y2-edition_chaine_y1)+1);

    largeur_max = edition_chaine_x2-edition_chaine_x1-1-2;
    longueur_chaine = strlen(edition_chaine_editee);
    i = 0;
    while ( (text_length(&edition_chaine_editee[i]) > largeur_max)
        && (i < longueur_chaine) )
        i++;
    write_xy_s_c_o_brut(ECRAN_REEL, edition_chaine_x1+1, edition_chaine_y1+1,
        COUL_OPTMENU_ACT_LIBELLE, -1, COUL_OPTMENU_ACT_OMBRE_LIB,
        &edition_chaine_editee[i]);
    edition_chaine_x_curseur = edition_chaine_x1+text_length(&edition_chaine_editee[i])+2;
}

/*
 * debut_edition_chaine: edite une chaine pour une option de type chaine
 *                       renvoie 1 si la chaine a ete validee et changee
 *                       sinon renvoie 0
 */

void debut_edition_chaine(int x1, int y1, int x2, int y2, unsigned char *defaut,
                          int longueur_max)

{
    int         i, j;
    t_sprite    spr_temp;

    if ( (sb16) && (sons_menu) )
        SON_DEBUT_EDITION_CHAINE;
    strcpy(edition_chaine_defaut, defaut);
    strcpy(edition_chaine_editee, defaut);
    getsprite(ECRAN_REEL, x1-1, y1, (x2-x1)+3, (y2-y1)+2, &fond_edition);
    putsprite(VSCR_AUTRES+3, &fond_edition, x1-1, y1);
    copy_v2v_xy(VSCR_AUTRES, VSCR_AUTRES+3, x1, y1, (x2-x1)+1, 1);
    for (i = x1; i <= x2; i++)
        for (j = y1; j <= y2; j++)
            assombrit_pixel(VSCR_AUTRES+3, ECRAN_REEL, i, j, 159, 159, 5);
    dessin_cadre_option(pos_option[pos_menu][pos_advanced][pos_barre][0],
        pos_option[pos_menu][pos_advanced][pos_barre][1],
        largeur_option[pos_menu][pos_advanced][pos_barre],
        5+(y2-y1)+1);
    getsprite(ECRAN_REEL, x1, y1, (x2-x1)+1, (y2-y1)+1, &spr_temp);
    putsprite(VSCR_AUTRES+3, &spr_temp, x1, y1);
    free(spr_temp.s_ptr);
    edition_chaine = 1;
    edition_chaine_x1 = x1;
    edition_chaine_y1 = y1;
    edition_chaine_x2 = x2;
    edition_chaine_y2 = y2;
    edition_chaine_max = longueur_max;
    edition_chaine_affiche(0);
}

/*
 * edition_chaine_traite_touche: traite la touche frappee pendant l'edition
 *                               la touche peut etre un caractere ou une
 *                               touche speciale (backspace, home, end, ...)
 */

void edition_chaine_traite_touche(int scancode)
{
    unsigned char   code_ascii;
    int             modif, i;

    modif = 0;
    if ( (scancode == 14) || (scancode == SCANCODE_CURSORBLOCKLEFT) )
    {
        if (strlen(edition_chaine_editee) > 0)
        {
            edition_chaine_editee[strlen(edition_chaine_editee)-1] = '\0';
            modif = 1;
        }
    }
    else
    {
        code_ascii = scancode_2_ascii(scancode);
        if (code_ascii != 0)
        {
            i = strlen(edition_chaine_editee);
            if (i < edition_chaine_max)
            {
                edition_chaine_editee[i] = code_ascii;
                edition_chaine_editee[i+1] = '\0';
                modif = 1;
            }
        }
    }
    if (modif)
        edition_chaine_affiche(1);
}

/*
 * fin_edition_chaine: termine l'edition d'une chaine
 */

void fin_edition_chaine()
{
    if ( (sb16) && (sons_menu) )
        SON_FIN_EDITION_CHAINE;
    putsprite(ECRAN_REEL, &fond_edition, edition_chaine_x1-1, edition_chaine_y1);
    free(fond_edition.s_ptr);
    edition_chaine = 0;
    affiche_option(ECRAN_REEL, pos_menu, pos_advanced, pos_barre, 1, 1, 2);
}

/*
 * affiche_volume: affiche le volume de sortie sur la SoundBlaster
 */

void affiche_volume(int no_vscr)
{
    int             i, j, nb_lignes;
    unsigned char   texte[10], chaine[50];

    if (sb16)
    {
        nb_lignes = (volume*100)/VOLUME_MAX;
        for (i = 0; i < nb_lignes; i++)
            ligne_vertic(no_vscr, X_VOLUME+i+1, Y_VOLUME+1, Y_VOLUME+HAUTEUR_VOLUME, COUL_FLAMES+((i*64)/100));
        if (nb_lignes < 100)
        {
            if (no_vscr == ECRAN_REEL)
                copy_v2s_xy(VSCR_MENU, X_VOLUME+nb_lignes+1, Y_VOLUME+1, 100-nb_lignes, HAUTEUR_VOLUME);
            else
                copy_v2v_xy(VSCR_MENU, no_vscr, X_VOLUME+nb_lignes+1, Y_VOLUME+1, 100-nb_lignes, HAUTEUR_VOLUME);
        }
        if (no_vscr == ECRAN_REEL)
            copy_v2s_xy(VSCR_MENU, X_VOLUME+86, Y_VOLUME-7, 12, 5);
        else
            copy_v2v_xy(VSCR_MENU, no_vscr, X_VOLUME+86, Y_VOLUME-7, 12, 5);
        sprintf(texte, "%3d", (volume*100)/VOLUME_MAX);
        chaine[1] = '\0';
        for (i = 0; i < 3; i++)
        {
            chaine[0] = texte[i];
            j = (chaine[0] == '1')?1:0;
            write_xy_s_c(no_vscr, X_VOLUME+86+(i*4)+j, Y_VOLUME-7, COUL_FLAMES+9+(volume/10), 4, chaine);
        }
    }
    else
    {
        if (no_sound)
            strcpy(chaine, "Sound/music disabled");
        else
            strcpy(chaine, "No SB16 detected!");
        write_xy_s_c(no_vscr, X_VOLUME+(101-text_length(chaine))/2,
            Y_VOLUME+1, COUL_FLAMES+9, 4, chaine);
    }
}

/*
 * efface_pos_musique: efface la position dans la musique
 */

void efface_pos_musique(int no_vscr)
{
    if (no_vscr == ECRAN_REEL)
        copy_v2s_xy(VSCR_MENU, X_VOLUME+1, Y_VOLUME+7, 100, 2);
    else
        copy_v2v_xy(VSCR_MENU, no_vscr, X_VOLUME+1, Y_VOLUME+7, 100, 2);
}

/*
 * affiche_pos_musique: affiche la position dans la musique
 */

void affiche_pos_musique(int no_vscr, int force_affichage)
{
    int             i, nb_lignes;

    if ( (sb16) && (row != 64)
        && ( ( (en_pause) && (music_jeu != NO_MUSIC) )
          || ( (!en_pause) && (music_menu != NO_MUSIC) ) ) )
    {
        if (orderpos <= 0)
            nb_lignes = 0;
        else
            nb_lignes = ((((orderpos-1)*64)+row)*100)/((orderlen*64)-1);
        if ( (nb_lignes != pos_musique) || (force_affichage) )
        {
            for (i = 0; i < nb_lignes; i++)
                ligne_vertic(no_vscr, X_VOLUME+i+1, Y_VOLUME+7, Y_VOLUME+8, COUL_FLAMES+((i*64)/100));
            if ( (nb_lignes < 100) && (nb_lignes < pos_musique) )
            {
                if (no_vscr == ECRAN_REEL)
                    copy_v2s_xy(VSCR_MENU, X_VOLUME+nb_lignes+1, Y_VOLUME+7, 100-nb_lignes, 2);
                else
                    copy_v2v_xy(VSCR_MENU, no_vscr, X_VOLUME+nb_lignes+1, Y_VOLUME+7, 100-nb_lignes, 2);
            }
            pos_musique = nb_lignes;
        }
    }
}

/*
 * affiche_fond_menu: affiche le fond d'une partie du menu
 */

void affiche_fond_menu(int no_vscr, int partie_menu, int advanced)
{
    switch (partie_menu)
    {
        case 0: /* haut */
            if (advanced)
            {
                if (no_vscr == ECRAN_REEL)
                {
                    copy_v2s_xy(VSCR_AUTRES, 96, 0, 224, 60-9);
                    copy_v2s_xy(VSCR_AUTRES, 96, 51, 128, 89);
                }
                else
                {
                    copy_v2v_xy(VSCR_AUTRES, no_vscr, 96, 0, 224, 60-9);
                    copy_v2v_xy(VSCR_AUTRES, no_vscr, 96, 51, 128, 89);
                }
            }
            else
            {
                if (no_vscr == ECRAN_REEL)
                    copy_v2s_xy(VSCR_AUTRES, 96, 0, 224, 60-9);
                else
                    copy_v2v_xy(VSCR_AUTRES, no_vscr, 96, 0, 224, 60-9);
            }
            break;
        case 1: /* droite */
            if (no_vscr == ECRAN_REEL)
                copy_v2s_xy(VSCR_AUTRES, 224+9, 60, 96-9, 140);
            else
                copy_v2v_xy(VSCR_AUTRES, no_vscr, 224+9, 60, 96-9, 140);
            break;
        case 2: /* bas */
            if (no_vscr == ECRAN_REEL)
                copy_v2s_xy(VSCR_AUTRES, 0, 140+9, 224, 60-9);
            else
                copy_v2v_xy(VSCR_AUTRES, no_vscr, 0, 140+9, 224, 60-9);
            break;
        case 3: /* gauche */
            if (no_vscr == ECRAN_REEL)
                copy_v2s_xy(VSCR_AUTRES, 0, 0, 96-9, 140);
            else
                copy_v2v_xy(VSCR_AUTRES, no_vscr, 0, 0, 96-9, 140);
            break;
    }
}

/*void affiche_fond_menu(int partie_menu, int actif)
{
    int         i, j;
    t_sprite    *ptr_fond;

    ptr_fond = (actif)?&grillage_clair:&grillage;
    switch (partie_menu)
    {
        case 0: // haut
            for (i = 0; i < 14; i++)
                for (j = 0; j < 3; j++)
                    putsprite(ptr_fond, 96+i*16, j*16);
            for (i = 0; i < 14; i++) // finition en bas
                putsprite_clip(ptr_fond, 96+i*16, 3*16, 0, 0, 16, 4);
            break;
        case 1: // droite
            for (i = 0; i < 5; i++)
                for (j = 0; j < 8; j++)
                    putsprite(ptr_fond, 232+i*16, 60+j*16);
            for (i = 0; i < 5; i++) // finition en bas
                putsprite_clip(ptr_fond, 232+i*16, 60+8*16, 0, 0, 16, 12);
            for (j = 0; j < 8; j++) // finition a droite
                putsprite_clip(ptr_fond, 232+5*16, 60+j*16, 0, 0, 8, 16);
            putsprite_clip(ptr_fond, 232+5*16, 60+8*16, 0, 0, 8, 12); // finition en bas a droite
            break;
        case 2: // bas
            for (i = 0; i < 14; i++)
                for (j = 0; j < 3; j++)
                    putsprite(ptr_fond, i*16, 149+j*16);
            for (i = 0; i < 14; i++) // finition en bas
                putsprite_clip(ptr_fond, i*16, 149+3*16, 0, 0, 16, 4);
            break;
        case 3: // gauche
            for (i = 0; i < 5; i++)
                for (j = 0; j < 8; j++)
                    putsprite(ptr_fond, i*16, j*16);
            for (i = 0; i < 5; i++) // finition en bas
                putsprite_clip(ptr_fond, i*16, 8*16, 0, 0, 16, 12);
            for (j = 0; j < 8; j++) // finition a droite
                putsprite_clip(ptr_fond, 5*16, j*16, 0, 0, 8, 16);
            putsprite_clip(ptr_fond, 5*16, 8*16, 0, 0, 8, 12); // finition en bas a droite
            break;
    }
}*/

/*
 * affiche_partie_menu: affiche une partie du menu (fond + texte)
 */

void affiche_partie_menu(int no_vscr, int partie_menu, int advanced,
                         int menu_actif)
{
    int     i, x, y, coul_titre, coul_degrade_titre, coul_ombre_titre;

    if (menu_actif)
    {
        coul_titre = 88;
        coul_degrade_titre = 2;
        coul_ombre_titre = 107; /* 26; */
    }
    else
    {
        coul_titre = 104;
        coul_degrade_titre = 0;
        coul_ombre_titre = 30;
    }
    switch (partie_menu)
    {
        case 0: /* haut -- FLASHTRIS */
            if (advanced)
            {
                x = 103;
                y = /*3*/ 6;
                for (i = 0; i < nb_options[0][advanced]; i++)
                {
                    pos_option[0][advanced][i][0] = x;
                    pos_option[0][advanced][i][1] = y;
                    largeur_option[0][advanced][i] = 96;
                    y += (affiche_option(no_vscr, 0, advanced, i,
                        menu_actif, (pos_barre == i), 0) & 255)+7;
                    /*if (y > 44)*/
                    if (y > 128)
                    {
                        x = 213;
                        y = 3;
                    }
                }
            }
            else
            {
                x = 100;
                for (i = 0; i < nb_options[0][advanced]; i++)
                {
                    pos_option[0][advanced][i][0] = x;
                    pos_option[0][advanced][i][1] = 41;
                    largeur_option[0][advanced][i] = 0;
                    x += (affiche_option(no_vscr, 0, advanced, i,
                        menu_actif, (pos_barre == i), 0) >> 8)+2;
                    if (langue == ANGLAIS)
                        x += 4;
                }
            }
            break;
        case 1: /* droite -- GAME */
            if (langue == ANGLAIS)
                write_xy_b_c_o(ECRAN_REEL, 241, 62, coul_titre, coul_degrade_titre, coul_ombre_titre, "GAME");
            else
                write_xy_b_c_o(ECRAN_REEL, 249, 62, coul_titre, coul_degrade_titre, coul_ombre_titre, "JEU");
            y = 81;
            for (i = 0; i < nb_options[1][advanced]; i++)
            {
                pos_option[1][advanced][i][0] = 236;
                pos_option[1][advanced][i][1] = y;
                largeur_option[1][advanced][i] = 80;
                y += (affiche_option(no_vscr, 1, advanced, i,
                    menu_actif, (pos_barre == i), 0) & 255)+5;
            }
            break;
        case 2: /* bas -- SOUND */
            if (langue == ANGLAIS)
                write_xy_b_c_o(ECRAN_REEL, 3, 151, coul_titre, coul_degrade_titre, coul_ombre_titre, "SOUND");
            else
                write_xy_b_c_o(ECRAN_REEL, 3, 151, coul_titre, coul_degrade_titre, coul_ombre_titre, "SON");
            x = 4;
            y = 170;
            for (i = 0; i < nb_options[2][advanced]; i++)
            {
                pos_option[2][advanced][i][0] = x;
                pos_option[2][advanced][i][1] = y;
                largeur_option[2][advanced][i] = 90; /* 106; --> trop large ! */
                y += (affiche_option(no_vscr, 2, advanced, i,
                    menu_actif, (pos_barre == i), 1) & 255)+6;
                if (y > 193)
                {
                    x += 100;
                    y = 154;
                }
            }
            /* dessin du volume et de la barre d'avancement de la musique */
            if (menu_actif)
                write_xy_s_c_o(ECRAN_REEL, X_VOLUME, Y_VOLUME-7, 73, -1, 28, "VOLUME");
            else
                write_xy_s_c_o(ECRAN_REEL, X_VOLUME, Y_VOLUME-7, 78, -1, 28, "VOLUME");
            for (i = X_VOLUME+1; i < X_VOLUME+101; i++)
            {
                putpixel_ofs(ECRAN_REEL, i+(Y_VOLUME*scr_nbx), COUL_FLAMES+63-(((i-X_VOLUME-1)*64)/100));
                putpixel_ofs(ECRAN_REEL, i+((Y_VOLUME+HAUTEUR_VOLUME+1)*scr_nbx), COUL_FLAMES+63-(((i-X_VOLUME-1)*64)/100));
                putpixel_ofs(ECRAN_REEL, i+((Y_VOLUME+9)*scr_nbx), COUL_FLAMES+63-(((i-X_VOLUME-1)*64)/100));
            }
            ligne_vertic(ECRAN_REEL, X_VOLUME, Y_VOLUME, Y_VOLUME+HAUTEUR_VOLUME+4, COUL_FLAMES+63);
            ligne_vertic(ECRAN_REEL, X_VOLUME+101, Y_VOLUME, Y_VOLUME+HAUTEUR_VOLUME+4, COUL_FLAMES);
            if (sb16)
                write_xy_s_c(ECRAN_REEL, X_VOLUME+98, Y_VOLUME-7, COUL_FLAMES+9, 4, "%%");
            affiche_volume(ECRAN_REEL);
            break;
        case 3: /* gauche -- NET */
            write_xy_b_c_o(ECRAN_REEL, 21, 2, coul_titre, coul_degrade_titre, coul_ombre_titre, "NET");
            y = 22;
            for (i = 0; i < nb_options[3][advanced]; i++)
            {
                pos_option[3][advanced][i][0] = 3;
                pos_option[3][advanced][i][1] = y;
                largeur_option[3][advanced][i] = 80;
                y += (affiche_option(no_vscr, 3, advanced, i,
                    menu_actif, (pos_barre == i), 0) & 255)+6;
            }
            break;
    }
}

/*
 * rotation_palette_grillage
 */

/*void rotation_palette_grillage()
{
    unsigned char   pal_temp[3];
    int             i;

    compteur_palette++;
    if (compteur_palette == 20)
    {
        compteur_palette = 0;
        memcpy(pal_temp, palette+251*3, 3);
        memmove(palette+251*3, palette+252*3, 3*3);
        memcpy(palette+254*3, pal_temp, 3);
        for (i = 251; i <= 254; i++)
            setcoul(i, palette[i*3], palette[i*3+1], palette[i*3+2]);
    }
}*/

/*
 * affiche_bouton: affiche un bouton de menu
 */

/*void affiche_bouton(int x, int y, int coul_texte, int coul_fond,
                    int plus_x, int plus_y, int smallfont, unsigned char *libelle)
{
    int     x2, y2, longueur, hauteur, i, j;

    copy_s2v(vscreen);
    if (smallfont)
    {
        longueur = text_length(libelle);
        hauteur = 5;
    }
    else
    {
        longueur = 9*strlen(libelle);
        hauteur = 8;
    }

    x2 = x+longueur+3+plus_x*2;
    y2 = y+hauteur+3+plus_y*2;

    coul_fond = 100;

    for (i = x+2; i <= x2-1; i++)
        for (j = y+2; j <= y2-2; j++)
            putpixel_ofs(i+j*scr_nbx, coul_fond+vscreen[i+j*scr_nbx]);
    //fill_rectangle(x+2, y+2, x2-2, y2-2, coul_fond);

    // 2 lignes horizontales du haut
    for (i = x; i <= x2-1; i++)
        putpixel_ofs(i+y*scr_nbx, coul_fond+2+vscreen[i+y*scr_nbx]);
    //ligne_horiz(x, x2-1, y, coul_fond+2);
    for (i = x+1; i <= x2-1; i++)
        putpixel_ofs(i+(y+1)*scr_nbx, coul_fond+1+vscreen[i+(y+1)*scr_nbx]);
    //ligne_horiz(x+1, x2-1, y+1, coul_fond+1);
    // 2 lignes verticales de gauche
    for (j = y+1; j <= y2-1; j++)
        putpixel_ofs(x+j*scr_nbx, coul_fond+2+vscreen[x+j*scr_nbx]);
    //ligne_vertic(x, y+1, y2-1, coul_fond+2);
    for (j = y+2; j <= y2-2; j++)
        putpixel_ofs(x+1+j*scr_nbx, coul_fond+1+vscreen[x+1+j*scr_nbx]);
    //ligne_vertic(x+1, y+2, y2-2, coul_fond+1);
    // 2 lignes horizontales du bas
    for (i = x+1; i <= x2; i++)
        putpixel_ofs(i+y2*scr_nbx, coul_fond-2+vscreen[i+y2*scr_nbx]);
    //ligne_horiz(x+1, x2, y2, coul_fond-2);
    for (i = x+2; i <= x2-1; i++)
        putpixel_ofs(i+(y2-1)*scr_nbx, coul_fond-1+vscreen[i+(y2-1)*scr_nbx]);
    //ligne_horiz(x+2, x2-1, y2-1, coul_fond-1);
    // 2 lignes verticales de droite
    for (j = y+1; j <= y2-1; j++)
        putpixel_ofs(x2+j*scr_nbx, coul_fond-2+vscreen[x2+j*scr_nbx]);
    //ligne_vertic(x2, y+1, y2-1, coul_fond-2);
    for (j = y+2; j <= y2-2; j++)
        putpixel_ofs(x2-1+j*scr_nbx, coul_fond-1+vscreen[x2-1+j*scr_nbx]);
    //ligne_vertic(x2-1, y+2, y2-2, coul_fond-1);

    putpixel_ofs(x+y2*scr_nbx, coul_fond+vscreen[x+y2*scr_nbx]);
    //putpixel_ofs(x+y2*scr_nbx, coul_fond);
    putpixel_ofs(x+1+(y2-1)*scr_nbx, coul_fond+vscreen[x+1+(y2-1)*scr_nbx]);
    //putpixel_ofs(x+1+(y2-1)*scr_nbx, coul_fond);

    putpixel_ofs(x2+y*scr_nbx, coul_fond+vscreen[x2+y*scr_nbx]);
    //putpixel_ofs(x2+y*scr_nbx, coul_fond);
    putpixel_ofs(x2-1+(y+1)*scr_nbx, coul_fond+vscreen[x2-1+(y+1)*scr_nbx]);
    //putpixel_ofs(x2-1+(y+1)*scr_nbx, coul_fond);

    if (smallfont)
        write_xy_s_c(ECRAN_REEL, x+plus_x+2, y+plus_y+2, coul_texte, 1, libelle);
    else
        write_xy_c(ECRAN_REEL, x+plus_x+2, y+plus_y+2, coul_texte, 0, libelle);
}*/

        /*// VERT:
        for (i = 96; i < 96+8; i++)
            setcoul(i, 48-(i-96)*6, 63, 48-(i-96)*6);
        for (i = 96+8; i < 96+8+52; i++)
            setcoul(i, 0, 63-(i-96-8), 0);*/
        /*// BLEU:
        for (i = 146; i < 146+8; i++)
            setcoul(i, 48-(i-146)*6, 48-(i-146)*6, 63);
        for (i = 146+8; i < 146+8+52; i++)
            setcoul(i, 0, 0, 63-(i-146-8));*/
        /*// ROUGE:
        for (i = 146; i < 146+8; i++)
            setcoul(i, 63, 48-(i-146)*6, 48-(i-146)*6);
        for (i = 146+8; i < 146+8+52; i++)
            setcoul(i, 63-(i-146-8), 0, 0);*/

        /*// ROUGE - VERT - BLEU
        for (i = 146; i < 146+16; i++)
            setcoul(i, 63-(i-146)*4, (i-146)*4, 0);
        for (i = 146+16; i < 146+32; i++)
            setcoul(i, 0, 63-(i-146-16)*4, (i-146-16)*4);
        for (i = 146+32; i < 146+48; i++)
            setcoul(i, (i-146-32)*4, 0, 63-(i-146-32)*4);*/


/*
 * init_flames: initialise les flames
 */

void init_flames(int x1, int y1, int coul, int nb_coul, t_sprite *sprite_image)
{
    int     i;

    /*  pas de flames sous X-Window !!!  */
    flam_x1 = x1;
    flam_y1 = y1;
    flam_x2 = x1+sprite_image->s_lx-1;
    flam_y2 = y1+sprite_image->s_ly-1;
    flam_coul = coul;
    flam_nb_coul = nb_coul;
    flam_largeur = flam_x2-flam_x1+1;
    flam_hauteur = flam_y2-flam_y1+1+3;
    flam_taille_image = flam_largeur*flam_hauteur;

    flam_src = (unsigned char *)ftris_malloc(flam_taille_image+flam_largeur*2, "burning title");
    flam_dest = (unsigned char *)ftris_malloc(flam_taille_image+flam_largeur*2, "burning title");
    memset(flam_src, flam_coul+flam_nb_coul-1, flam_taille_image+flam_largeur*2);
    memset(flam_dest, flam_coul+flam_nb_coul-1, flam_taille_image+flam_largeur*2);

    /*copy_s2v(vscreen_menu);
    getsprite(vscreen_menu, flam_x1, flam_y1, flam_largeur, flam_hauteur-3, &flam_image);
    getsprite(vscreen_menu, flam_x1, flam_y1, flam_largeur, flam_hauteur-3, &flam_vscreen);*/
    flam_image = sprite_image;
    #ifndef MSDOS
    if (X_Window)
    {
        /*  X-Window  */
        /*getsprite(VSCR_VSCREEN, 0, 0, flam_image->s_lx, flam_image->s_ly,
            &flam_a_afficher);*/
        flam_a_afficher.s_lx = flam_image->s_lx;
        flam_a_afficher.s_ly = flam_image->s_ly;
        flam_a_afficher.s_ximage = XSubImage(flam_image->s_ximage, 0, 0,
            flam_image->s_lx, flam_image->s_ly);
    }
    else
    #endif
    {
        flam_a_afficher.s_lx = flam_image->s_lx;
        flam_a_afficher.s_ly = flam_image->s_ly;
        flam_a_afficher.s_ptr = (unsigned char *)ftris_malloc(flam_image->s_lx*flam_image->s_ly, "burning title");
        memcpy(flam_a_afficher.s_ptr, flam_image->s_ptr, flam_image->s_lx*flam_image->s_ly);
    }

    /* initialise les flames pour le copyright */
    flam_copyright.s_lx = copyright.s_lx;
    flam_copyright.s_ly = copyright.s_ly;
    flam_copyright.s_ptr = (unsigned char *)ftris_malloc(copyright.s_lx*copyright.s_ly, "burning title");
    memcpy(flam_copyright.s_ptr, copyright.s_ptr, copyright.s_lx*copyright.s_ly);

    /* ROUGE --> JAUNE */
    for (i = COUL_FLAMES; i < COUL_FLAMES+32; i++)
        setcoul(i, 63-(i-COUL_FLAMES)/4, (i-COUL_FLAMES)*2, 0);
    for (i = COUL_FLAMES+32; i < COUL_FLAMES+64; i++)
        setcoul(i, 56+(i-COUL_FLAMES-32)/4, 63, (i-COUL_FLAMES-32)*2);
}

/*
 * flames: enflamme le titre "FlashTris"...........
 */

void flames(int affiche)
{
    int             i, k, ptr_dest;
    long            j;
    unsigned char   *ptr_temp;

    /*  pas de flames sous X-Window !!!  */
    #ifndef MSDOS
    if (X_Window)
        return;
    #endif
    ptr_dest = 0;
    for (i = flam_largeur; i < flam_taille_image; i++)
    {
        j = (flam_src[i-flam_largeur-1]*2+flam_src[i-flam_largeur]*2+
            flam_src[i-flam_largeur+1]*2+flam_src[i-1]*5+flam_src[i+1]*5+
            flam_src[i+flam_largeur-1]*9+flam_src[i+flam_largeur]*9+
            flam_src[i+flam_largeur+1]*9)/43;
        /*j = (src[i-largeur-1]+src[i-largeur]+src[i-largeur+1]+
            src[i-1]+src[i+1]+src[i+largeur-1]+src[i+largeur]+
            src[i+largeur+1]) >> 3;*/
        if (j > flam_coul) j--;
        flam_dest[ptr_dest++] = j;
    }
    for (i = 0; i < flam_largeur; i++)
    {
        flam_dest[(flam_hauteur-1)*flam_largeur+i] = flam_coul+(rand()%flam_nb_coul);
        flam_dest[(flam_hauteur)*flam_largeur+i] = flam_coul+(rand()%flam_nb_coul);
        flam_dest[(flam_hauteur+1)*flam_largeur+i] = flam_coul+(rand()%flam_nb_coul);
    }
    for (i = 0; i < rand()%10; i++)
    {
        j = (flam_hauteur-3)*flam_largeur+(rand()%flam_largeur);
        flam_dest[j-flam_largeur-1] = flam_coul+flam_nb_coul-1;
        flam_dest[j-flam_largeur] = flam_coul+flam_nb_coul-1;
        flam_dest[j-flam_largeur+1] = flam_coul+flam_nb_coul-1;
        flam_dest[j-1] = flam_coul+flam_nb_coul-1;
        flam_dest[j] = flam_coul+flam_nb_coul-1;
        flam_dest[j+1] = flam_coul+flam_nb_coul-1;
        flam_dest[j+flam_largeur-1] = flam_coul+flam_nb_coul-1;
        flam_dest[j+flam_largeur] = flam_coul+flam_nb_coul-1;
        flam_dest[j+flam_largeur+1] = flam_coul+flam_nb_coul-1;
    }
    /*for (i = 0; i < hauteur; i++)
        memcpy(vscreen+x1+(y1+i)*scr_nbx, dest+i*largeur, largeur);*/

    /*__asm__ __volatile__ ("
        movl %0, %%esi\n
        movl %1, %%edi\n
        movl %2, %%ecx\n
        movl %%ecx, %%edx\n
    boucle_recopie_flame:
        cmpb $0, (%%edi)\n
        jz   pas_touche\n
    pas_touche:
        incl %%edi\n */

    for (j = 0; j < flam_hauteur-3; j++)
        for (i = 0; i < flam_largeur; i++)
        {
            k = i+j*flam_largeur;
            if (flam_image->s_ptr[k] != 0)
                flam_a_afficher.s_ptr[k] = flam_dest[k+flam_largeur];
            /*else
                if (vscr_menu[k+scr_nbx] != 0)
                    vscreen[k] = coul+nb_coul-1-(dest[i+j*largeur]-coul)/2;*/
        }

    if (affiche)
        putsprite_tr(&flam_a_afficher, flam_x1, flam_y1);
    else
        putsprite_v_tr(VSCR_VSCREEN, &flam_a_afficher, flam_x1, flam_y1);
    ptr_temp = flam_src;
    flam_src = flam_dest;
    flam_dest = ptr_temp;
}

void flames_volume()
{
    int     i, j;

    /*  pas de flames sous X-Window !!!  */
    #ifndef MSDOS
    if (X_Window)
        return;
    #endif
    for (i = 0; i < volume; i++)
        for (j = 0; j < 8; j++)
            putpixel_ofs(ECRAN_REEL, i+20+(170+j)*scr_nbx,
                flam_dest[i+flam_largeur*((flam_hauteur*j)/8)]);
}

void flames_copyright()
{
    int     i, j;

    /*  pas de flames sous X-Window !!!  */
    #ifndef MSDOS
    if (X_Window)
        return;
    #endif
    for (i = 0; i < 217; i++)
        for (j = 0; j < 12; j++)
            if (copyright.s_ptr[i+j*217] != 0)
                flam_copyright.s_ptr[i+j*217] =
                    flam_dest[i+flam_largeur*((flam_hauteur*j)/14)];
    /*copy_v2s_xy(vscreen, 0, 188, 217, 12);*/
    putsprite_tr(&flam_copyright, 0, 188);
}

/*
 * stop_flames: arrete les flames (snif!)
 */

void stop_flames()
{
    /*  pas de flames sous X-Window !!!  */
    #ifndef MSDOS
    if (X_Window)
        return;
    #endif
    free(flam_src);
    free(flam_dest);
    free(flam_a_afficher.s_ptr);
    free(flam_copyright.s_ptr);
}

/* ========================================================================== */

/*
 * init_scrolltxt: initialize le scrolling texte
 */

void init_scrolltxt()
{
    int     i;

    /*  pas de scrolling texte sous X-Window !!!  */
    #ifndef MSDOS
    if (X_Window)
        return;
    #endif
    scrolltxt_lx_image = text_length(texte_scrolling[langue]);
    image_scroll = (unsigned char *)ftris_malloc(scrolltxt_lx_image*5, "scrolling text");
    memset(image_scroll, 0, scrolltxt_lx_image*5);
    make_image_texte_s(image_scroll, scrolltxt_lx_image, texte_scrolling[langue], 15);
    memset(scrolltxt_coul, 15, SCROLLTXT_LARGEUR);
    /*for (i = 0; i < 16; i++)
    {
        scrolltxt_coul[i] = 31-i;
        scrolltxt_coul[SCROLLTXT_LARGEUR-i-1] = 31-i;
    }*/
    for (i = 0; i < 32; i+=2)
    {
        scrolltxt_coul[i] = 31-i/2;
        scrolltxt_coul[i+1] = 31-i/2;
        scrolltxt_coul[SCROLLTXT_LARGEUR-i-1] = 31-i/2;
        scrolltxt_coul[SCROLLTXT_LARGEUR-i-2] = 31-i/2;
    }
    scrolltxt_x = 0;
    scrolltxt_cptr = rand()%1500;
}

/*
 * scrolltxt: scrolling de texte (pour le copyright)
 */

void scrolltxt()
{
    int     i, j;

    /*  pas de scrolling texte sous X-Window !!!  */
    #ifndef MSDOS
    if (X_Window)
        return;
    #endif
    if (scrolltxt_cptr > 0)
        scrolltxt_cptr--;
    else
    {
        /*for (i = 0; i < 5; i++)
            memcpy_rapide(vscreen[VSCR_VSCREEN]+SCROLLTXT_OFS+i*scr_nbx,
                vscreen[VSCR_MENU]+SCROLLTXT_OFS+i*scr_nbx, (SCROLLTXT_LARGEUR/4)+1);*/
        copy_v2v_xy(VSCR_MENU, VSCR_VSCREEN, SCROLLTXT_X, SCROLLTXT_Y,
            SCROLLTXT_LARGEUR, 5);
        for (i = 0; i < SCROLLTXT_LARGEUR; i++)
            for (j = 0; j < 5; j++)
                if (image_scroll[scrolltxt_x+i+j*scrolltxt_lx_image] != 0)
                    putpixel_ofs(VSCR_VSCREEN, SCROLLTXT_OFS+i+j*scr_nbx, scrolltxt_coul[i]);
        copy_v2s_xy(VSCR_VSCREEN, SCROLLTXT_X, SCROLLTXT_Y, SCROLLTXT_LARGEUR, 5);
        scrolltxt_x++;
        if (scrolltxt_x == scrolltxt_lx_image-SCROLLTXT_LARGEUR)
        {
            scrolltxt_x = 0;
            scrolltxt_cptr = 1500;
        }
    }
}

/*
 * scroll_resauve_fond: reprend le fond de VSCR_AUTRES pour le remettre
 *                      dans vscr_menu (suite a un changement de fond du menu)
 */

void scroll_resauve_fond()
{
    /*  pas de scrolling texte sous X-Window !!!  */
    #ifndef MSDOS
    if (X_Window)
        return;
    #endif
    copy_v2v_xy(VSCR_AUTRES, VSCR_MENU, SCROLLTXT_X, SCROLLTXT_Y,
        SCROLLTXT_LARGEUR, 5);
}

/*
 * scroll_restaure_fond: restaure le fond du scrolling
 */

void scroll_restaure_fond()
{
    /*  pas de scrolling texte sous X-Window !!!  */
    #ifndef MSDOS
    if (X_Window)
        return;
    #endif
    copy_v2s_xy(VSCR_MENU, SCROLLTXT_X, SCROLLTXT_Y, SCROLLTXT_LARGEUR, 5);
}

/*
 * stop_scrolling: fin du scrolling
 */

void stop_scroll()
{
    /*  pas de scrolling texte sous X-Window !!!  */
    #ifndef MSDOS
    if (X_Window)
        return;
    #endif
    free(image_scroll);
}

/* ========================================================================== */

/*
 * init_brik_image: initialise l'image contenant les mots en briques
 */

void init_brik_image()
{
    int             i, j, k, l, pos_x, plus_a_droite;
    unsigned char   car;

    brik_image = (unsigned char *)ftris_malloc(BRIK_IMAGE_NBX*BRIK_IMAGE_NBY, "briks image");
    memset(brik_image, 0, BRIK_IMAGE_NBX*BRIK_IMAGE_NBY);
    getsprite(VSCR_MENU, BRIK_X, BRIK_Y, BRIK_NBX_AFF, BRIK_NBY_AFF, brik_fond);

    for (i = 0; i < BRIK_NB_LIG; i++)
    {
        pos_x = (BRIK_NBX_AFF-(text_length(brik_data[langue][i])*2))/2;
        for (j = 0; j < strlen(brik_data[langue][i]); j++)
        {
            if ( (brik_data[langue][i][j] >= 'a') && (brik_data[langue][i][j] <= 'z') )
                car = brik_data[langue][i][j]-('a'-'A')-33;
            else
                car = brik_data[langue][i][j]-33;
            plus_a_droite = 0;
            for (k = 0; k < 5; k++)
                for (l = 0; l < 5; l++)
                    if ((petite_fonte[car][k] & (16 >> l)) != 0)
                    {
                        brik_image[(BRIK_HAUT_LIG*i+k*2)*BRIK_IMAGE_NBX+pos_x+l*2] =
                            brik_coul[i]+k;
                        brik_image[(BRIK_HAUT_LIG*i+k*2)*BRIK_IMAGE_NBX+pos_x+l*2+1] = 1;
                        brik_image[(BRIK_HAUT_LIG*i+k*2)*BRIK_IMAGE_NBX+pos_x+l*2+BRIK_IMAGE_NBX] = 1;
                        brik_image[(BRIK_HAUT_LIG*i+k*2)*BRIK_IMAGE_NBX+pos_x+l*2+BRIK_IMAGE_NBX+1] = 1;
                        if (plus_a_droite < l)
                            plus_a_droite = l;
                    }
            pos_x += plus_a_droite*2+4;
        }
    }
}

/*
 * init_brik: initialise l'affichage des mots en briques
 */

void init_brik()
{
    brik_no_lig = 0;
    brik_cptr = (rand()%500)+200;
    brik_scroll_y = BRIK_HAUT_LIG*2;

    init_brik_image();
}

/*
 * brik_efface: efface le mot en brique
 */

void brik_efface()
{
    putsprite(ECRAN_REEL, brik_fond, BRIK_X, BRIK_Y);
}

/*
 * brik_affiche: affiche les briques (en tenant compte du scroll)
 */

void brik_affiche()
{
    int     i, j, position, ofs;

    position = brik_no_lig*BRIK_HAUT_LIG+(BRIK_HAUT_LIG-(brik_scroll_y/2));
    putsprite(VSCR_VSCREEN, brik_fond, BRIK_X, BRIK_Y);
    for (i = 0; i < BRIK_NBX_AFF; i++)
        for (j = 0; j < BRIK_NBY_AFF; j++)
        {
            ofs = (j+position)*BRIK_IMAGE_NBX+i;
            if (brik_image[ofs] > 1)
                putpixel_ofs(VSCR_VSCREEN, (BRIK_Y+j)*scr_nbx+BRIK_X+i,
                    brik_image[ofs]);
            else
                if (brik_image[ofs] == 1)
                    assombrit_pixel(VSCR_VSCREEN, VSCR_VSCREEN, BRIK_X+i,
                        BRIK_Y+j, 31, 0, 12);
        }
    copy_v2s_xy(VSCR_VSCREEN, BRIK_X, BRIK_Y, BRIK_NBX_AFF, BRIK_NBY_AFF);
}

/*
 * brik: affiche les mots en briques
 */

/*void brik()
{
    int     i, j;

    if (brik_nb_reste > 0)
    {
        // choisit un carre vide au hasard
        i = rand()%brik_nb_x;
        j = rand()%brik_nb_y;
        while (brik_image[i+j*BRIK_LX_MAX] != 0)
        {
            i++;
            if (i == brik_nb_x)
            {
                i = 0;
                j++;
                if (j == brik_nb_y)
                    j = 0;
            }
        }
        copy_v2s_xy(VSCR_MENU, BRIK_X+i*3, BRIK_Y+j*3, 3, 3);
        brik_image[i+j*BRIK_LX_MAX] = 2;
        brik_nb_reste--;
        if (brik_nb_reste == 0)
            brik_cptr = 100;
    }
    else
        if (brik_cptr > 0)
            brik_cptr--;
        else
        {
            if (brik_no_mot != -1)
            {
                // effacement de l'ancien mot
                copy_v2s_xy(VSCR_MENU, BRIK_X, BRIK_Y, brik_nb_x*3, brik_nb_y*3);
            }
            // nouveau mot
            brik_no_mot++;
            if (brik_mots[brik_no_mot] == NULL)
                brik_no_mot = 0;
            memset(brik_image, 0, BRIK_LX_MAX*BRIK_LY_MAX);
            make_image_texte_s(brik_image, BRIK_LX_MAX, brik_mots[brik_no_mot], 1);
            brik_nb_x = text_length(brik_mots[brik_no_mot]);
            brik_nb_y = 5;
            // compte le nombre de "trous" ( = nombre d'explosions a creer)
            brik_nb_reste = 0;
            for (i = 0; i < brik_nb_x; i++)
                for (j = 0; j < brik_nb_y; j++)
                {
                    if (brik_image[i+j*BRIK_LX_MAX] == 0)
                        brik_nb_reste++;
                    putsprite(ECRAN_REEL, &brique[3-TC_MIN][rand()%3], BRIK_X+i*3, BRIK_Y+j*3);
                }
        }
}*/

/*
 * brik: affiche les mots en briques
 */

void brik()
{
    if (brik_copyright)
    {
        if (brik_cptr > 0)
            brik_cptr--;
        else
            if (brik_scroll_y > 0)
            {
                brik_scroll_y--;
                if (brik_scroll_y%2 == 0)
                    brik_affiche();
                if (brik_scroll_y == 0)
                {
                    brik_scroll_y = BRIK_HAUT_LIG*2;
                    brik_no_lig++;
                    if (brik_no_lig == BRIK_NB_LIG-1)
                    {
                        brik_no_lig = 0;
                        brik_cptr = (rand()%2000)+1000;  /* retour au debut */
                    }
                    else
                        brik_cptr = brik_time[brik_no_lig]; /* temps pour voir la nouvelle ligne */
                }
            }
    }
}

/*
 * stop_brik: stoppe les mots en briques
 */

void stop_brik()
{
    free(brik_image);
    free(brik_fond->s_ptr);
}

/*
 * pixels_init: initialise les pixels
 */

void pixels_init()
{
    int     i;

    if (pixel_nb > 0)
    {
        for (i = 0; i < NB_PIXELS_MAX; i++)
        {
            pixel_x[i] = -1;
            pixel_y[i] = -1;
            pixel_fond[i] = -1;
        }
    }
}

/*
 * pixels_calcule_nb: calcule le nombre de pixels en fonction de la largeur de
 *                    l'option
 */

void pixels_calcule_nb()
{
    pixel_nb_a_afficher =
        ((largeur_option_reelle[pos_menu][pos_advanced][pos_barre]+10)*
         (pixel_nb_tab[pixel_nb]))/100;
    if (pixel_nb_a_afficher > NB_PIXELS_MAX)
        pixel_nb_a_afficher = NB_PIXELS_MAX;
}

/*
 * pixels_efface: efface tous les pixels
 */

void pixels_efface()
{
    int     i, ofs;

    if (pixel_nb > 0)
    {
        for (i = 0; i < pixel_nb_a_afficher; i++)
            if ( (pixel_x[i] >= 0) && (pixel_x[i] < scr_nbx)
                && (pixel_y[i] >= 0) && (pixel_y[i] < scr_nby)
                && (pixel_fond[i] != -1) )
            {
                ofs = (int)(pixel_x[i])+((int)(pixel_y[i]))*scr_nbx;
                putpixel_ofs(ECRAN_REEL, ofs, pixel_fond[i]);
            }
    }
}

/*
 * pixels_affiche: affiche tous les pixels
 */

void pixels_affiche()
{
    int     i, ofs;

    if (pixel_nb > 0)
    {
        for (i = 0; i < pixel_nb_a_afficher; i++)
            if ( (pixel_x[i] >= 0) && (pixel_x[i] < scr_nbx)
                && (pixel_y[i] >= 0) && (pixel_y[i] < scr_nby) )
            {
                ofs = (int)(pixel_x[i])+((int)(pixel_y[i]))*scr_nbx;
                putpixel_ofs(ECRAN_REEL, ofs, pixel_couleur[i]);
            }
    }
}

/*
 * pixels_sauve_fond: sauve le fond derriere les pixels
 */

void pixels_sauve_fond()
{
    int     i, ofs;

    if (pixel_nb > 0)
    {
        for (i = 0; i < pixel_nb_a_afficher; i++)
        {
            ofs = (int)(pixel_x[i])+((int)(pixel_y[i]))*scr_nbx;
            pixel_fond[i] = getpixel_ofs(ECRAN_REEL, ofs);
        }
    }
}

/*
 * pixels_anim: animation des pixels
 */

void pixels_anim()
{
    int     i, ofs;

    if (pixel_nb > 0)
    {
        pixels_efface();
        for (i = 0; i < pixel_nb_a_afficher; i++)
        {
            if ( (pixel_x[i] < 0) || (pixel_x[i] >= scr_nbx)
                || (pixel_y[i] < 0) || (pixel_y[i] >= scr_nby) )
            {
                pixel_x[i] = pos_option[pos_menu][pos_advanced][pos_barre][0]+
                    (rand()%(largeur_option_reelle[pos_menu][pos_advanced][pos_barre]));
                if (rand()%2 == 0)
                {
                    if ( (liste_choix) && (pixel_x[i] >= liste_choix_x1) )
                        pixel_y[i] = liste_choix_y1-1;
                    else
                        pixel_y[i] = pos_option[pos_menu][pos_advanced][pos_barre][1]-3+0.9;
                    pixel_inc_y[i] = -((float)(rand()%15))/((float)30);
                }
                else
                {
                    if (edition_chaine)
                        pixel_y[i] = pos_option[pos_menu][pos_advanced][pos_barre][1]+17;
                    else
                        if ( (liste_choix) && (pixel_x[i] >= liste_choix_x1) )
                            pixel_y[i] = liste_choix_y2+1;
                        else
                            pixel_y[i] = pos_option[pos_menu][pos_advanced][pos_barre][1]+8;
                    pixel_inc_y[i] = ((float)(rand()%15))/((float)30);
                }
                pixel_inc_x[i] = ((float)(rand()%15))/((float)15);
                if (rand()%2 == 0)
                    pixel_inc_x[i] = -pixel_inc_x[i];
                pixel_couleur[i] = 15;
                pixel_inc_coul[i] = (float)(0.5+rand()%3);
            }
            ofs = (int)(pixel_x[i])+((int)(pixel_y[i]))*scr_nbx;
            /*if (pixel_fond[i] != -1)
                putpixel_ofs(ECRAN_REEL, ofs, pixel_fond[i]);*/
            pixel_x[i] = pixel_x[i]+pixel_inc_x[i];
            pixel_y[i] = pixel_y[i]+pixel_inc_y[i];
            pixel_couleur[i] = pixel_couleur[i]+pixel_inc_coul[i];
            if ((int)(pixel_couleur[i]) > 31)
            {
                pixel_x[i] = -1;
                pixel_y[i] = -1;
                pixel_fond[i] = -1;
            }
            else
                if ( (pixel_x[i] >= 0) && (pixel_x[i] < scr_nbx)
                    && (pixel_y[i] >= 0) && (pixel_y[i] < scr_nby) )
                {
                    ofs = (int)(pixel_x[i])+((int)(pixel_y[i]))*scr_nbx;
                    pixel_fond[i] = getpixel_ofs(ECRAN_REEL, ofs);
                    //putpixel_ofs(ECRAN_REEL, ofs, pixel_couleur[i]);
                }
        }
        pixels_affiche();
    }
}

/* ========================================================================== */

/*
 * affiche_brique_menu: affiche une petite brique pour le menu
 */

/*void affiche_brique_menu(unsigned char *scr, int x, int y, int taille,
                         unsigned char couleur)
{
    #ifndef MSDOS
    if (X_Window)
    {
        //  X-Window: attention a scr 
        XSetForeground(display, gc, xcolor[couleur]);
        XFillRectangle(display, window, gc, x, y, taille, taille);
    }
    else
    #endif
    // Linux Console ou Dos
    {
        //x2 = x+taille-1;
        //for (i = 0; i < taille; i++)
        //    ligne_horiz(x, x2, y+i, couleur);
        putsprite(scr, &brique_menu[taille-TC_MIN][couleur], x, y);
    }
}*/

/*
 * efface_piece_preview: efface une piece d'un joueur dans l'ecran de preview
 */

void efface_piece_preview(int no_vscr, int joueur, t_piece *p)
{
    int     i, x1, y1;

    x1 = pv_x1[joueur]+((pv_larg_aire-(4*pv_tc_piece))*(p->p_x-x_min[joueur]))/(nbx-5);
    y1 = pv_y1[joueur]+((pv_haut_aire/*-pv_tc_piece*/)*(p->p_y-y_min[joueur]))/nby;
    for (i = 0; i < 16; i++)
        if ( ((p->p_forme & (1 << i)) != 0)
            && (p->p_y+y_inc[i] >= y_min[joueur]) )
            efface_brique(no_vscr, VSCR_MENU,
                x1+x_inc[i]*pv_tc_piece,
                y1+y_inc[i]*pv_tc_piece,
                pv_tc_piece);
}

/*
 * affiche_piece_preview: affiche une piece d'un joueur dans l'ecran de preview
 */

void affiche_piece_preview(int no_vscr, int joueur, t_piece *p)
{
    int     i, x1, y1;

    x1 = pv_x1[joueur]+((pv_larg_aire-(4*pv_tc_piece))*(p->p_x-x_min[joueur]))/(nbx-5);
    y1 = pv_y1[joueur]+((pv_haut_aire/*-pv_tc_piece*/)*(p->p_y-y_min[joueur]))/nby;
    switch (p->p_bonus)
    {
        case PIECE_BOMBE:
            putsprite(no_vscr, &bombe[pv_tc_piece-TC_MIN][image_bombe/10],
                x1+pv_tc_piece, y1+pv_tc_piece);
            break;
        default:
            for (i = 0; i < 16; i++)
                if ( ((p->p_forme & (1 << i)) != 0)
                    && (p->p_y+y_inc[i] >= y_min[joueur]) )
                    affiche_brique(no_vscr,
                        x1+x_inc[i]*pv_tc_piece,
                        y1+y_inc[i]*pv_tc_piece,
                        pv_tc_piece,
                        p->p_couleur);
    }
}

/*
 * affiche_piece_suivante_preview: affiche la piece suivante d'un joueur
 *                                 dans l'ecran de preview
 */

void affiche_piece_suivante_preview(int no_vscr, int joueur, t_piece *p)
{
    int     i;

    if (get_game_team())
    {
        if (no_vscr == ECRAN_REEL)
            copy_v2s_xy(VSCR_MENU,
                pv_x1[joueur]+((x_next[joueur]-x_deb[team[joueur]])*2)/5+1,
                pv_y1[joueur]-pv_tc_next*4-2+1, pv_tc_next*4, pv_tc_next*4);
        else
            copy_v2v_xy(VSCR_MENU, no_vscr,
                pv_x1[joueur]+((x_next[joueur]-x_deb[team[joueur]])*2)/5+1,
                pv_y1[joueur]-pv_tc_next*4-2+1, pv_tc_next*4, pv_tc_next*4);
        switch (p->p_bonus)
        {
            case PIECE_BOMBE:
                putsprite(no_vscr, &bombe[pv_tc_next-TC_MIN][image_bombe/10],
                    pv_x1[joueur]+((x_next[joueur]-x_deb[team[joueur]])*2)/5+1+pv_tc_next,
                    pv_y1[joueur]-pv_tc_next*4-2+1+pv_tc_next);
                break;
            default:
                for (i = 0; i < 16; i++)
                    if ( (p->p_forme & (1 << i)) != 0 )
                        affiche_brique(no_vscr,
                            pv_x1[joueur]+((x_next[joueur]-x_deb[team[joueur]])*2)/5+1+x_inc[i]*pv_tc_next,
                            pv_y1[joueur]-pv_tc_next*4-2+1+y_inc[i]*pv_tc_next,
                            pv_tc_next,
                            p->p_couleur);
        }
    }
    else
    {
        if (no_vscr == ECRAN_REEL)
            copy_v2s_xy(VSCR_MENU, pv_x1[joueur]+pv_larg_aire+2,
                pv_y1[joueur]+1, pv_tc_next*4, pv_tc_next*4);
        else
            copy_v2v_xy(VSCR_MENU, no_vscr, pv_x1[joueur]+pv_larg_aire+2,
                pv_y1[joueur]+1, pv_tc_next*4, pv_tc_next*4);
        switch (p->p_bonus)
        {
            case PIECE_BOMBE:
                putsprite(no_vscr, &bombe[pv_tc_next-TC_MIN][image_bombe/10],
                    pv_x1[joueur]+pv_larg_aire+2+pv_tc_next,
                    pv_y1[joueur]+1+pv_tc_next);
                break;
            default:
                for (i = 0; i < 16; i++)
                    if ( (p->p_forme & (1 << i)) != 0 )
                        affiche_brique(no_vscr,
                            pv_x1[joueur]+pv_larg_aire+2+x_inc[i]*pv_tc_next,
                            pv_y1[joueur]+1+y_inc[i]*pv_tc_next,
                            pv_tc_next,
                            p->p_couleur);
        }
    }
}

/*
 * affiche_piece_suivante2_preview: affiche la 2eme piece suivante d'un joueur
 *                                  dans l'ecran de preview
 */

void affiche_piece_suivante2_preview(int no_vscr, int joueur, t_piece *p)
{
    int     i;

    if (get_game_team())
    {
        if (no_vscr == ECRAN_REEL)
            copy_v2s_xy(VSCR_MENU, pv_x1[joueur]+((x_next2[joueur]-x_deb[team[joueur]])*2)/5+1,
                pv_y1[joueur]-pv_tc_next*4-2-pv_tc_next2*4-2+1,
                pv_tc_next2*4, pv_tc_next2*4);
        else
            copy_v2v_xy(VSCR_MENU, no_vscr, pv_x1[joueur]+((x_next2[joueur]-x_deb[team[joueur]])*2)/5+1,
                pv_y1[joueur]-pv_tc_next*4-2-pv_tc_next2*4-2+1,
                pv_tc_next2*4, pv_tc_next2*4);
        switch (p->p_bonus)
        {
            case PIECE_BOMBE:
                putsprite(no_vscr, &bombe[pv_tc_next-TC_MIN][image_bombe/10],
                    pv_x1[joueur]+((x_next2[joueur]-x_deb[team[joueur]])*2)/5+1+pv_tc_next2,
                    pv_y1[joueur]-pv_tc_next*4-2-pv_tc_next2*4-2+1);
                break;
            default:
                for (i = 0; i < 16; i++)
                    if ( (p->p_forme & (1 << i)) != 0 )
                        affiche_brique(no_vscr,
                            pv_x1[joueur]+((x_next2[joueur]-x_deb[team[joueur]])*2)/5+1+x_inc[i]*pv_tc_next2,
                            pv_y1[joueur]-pv_tc_next*4-2-pv_tc_next2*4-2+1+y_inc[i]*pv_tc_next2,
                            pv_tc_next2,
                            p->p_couleur);
        }
    }
    else
    {
        if (no_vscr == ECRAN_REEL)
            copy_v2s_xy(VSCR_MENU, pv_x1[joueur]+pv_larg_aire+2,
                pv_y1[joueur]+pv_tc_next*4+4,
                pv_tc_next2*4, pv_tc_next2*4);
        else
            copy_v2v_xy(VSCR_MENU, no_vscr, pv_x1[joueur]+pv_larg_aire+2,
                pv_y1[joueur]+pv_tc_next*4+4,
                pv_tc_next2*4, pv_tc_next2*4);
        switch (p->p_bonus)
        {
            case PIECE_BOMBE:
                putsprite(no_vscr, &bombe[pv_tc_next-TC_MIN][image_bombe/10],
                    pv_x1[joueur]+pv_larg_aire+2+pv_tc_next2,
                    pv_y1[joueur]+pv_tc_next*4+4+pv_tc_next2);
                break;
            default:
                for (i = 0; i < 16; i++)
                    if ( (p->p_forme & (1 << i)) != 0 )
                        affiche_brique(no_vscr,
                            pv_x1[joueur]+pv_larg_aire+2+x_inc[i]*pv_tc_next2,
                            pv_y1[joueur]+pv_tc_next*4+4+y_inc[i]*pv_tc_next2,
                            pv_tc_next2,
                            p->p_couleur);
        }
    }
}

/*
 * affiche_pieces_suivantes_preview: affiche la/les pieces suivante(s) de la preview
 */

void affiche_pieces_suivantes_preview()
{
    int     i;

    /* affiche la/le(s) piece(s) suivante(s) */
    if (view_next > 0)
    {
        if (view_next == 2)
            for (i = 0; i < nbj; i++)
            {
                affiche_piece_suivante_preview(ECRAN_REEL, i, &next_piece[i]);
                affiche_piece_suivante2_preview(ECRAN_REEL, i, &next_piece2[i]);
            }
        else
            for (i = 0; i < nbj; i++)
                affiche_piece_suivante_preview(ECRAN_REEL, i, &next_piece[i]);
    }
}

/*
 * redessine_preview: redessine la previsualisation de l'ecran de jeu
 *                    si une option de jeu a ete modifiee
 */

void redessine_preview(int nouvelles_pieces, int nouveau_handicap)
{
    int     i;

    /* dessin de l'ecran de jeu */
    calcule_pos_joueurs();
    if (nouvelles_pieces)
    {
        init_jeu();
        /* creation des pieces d'exemple */
        for (i = 0; i < nbj; i++)
            nouvelle_piece(i, 0);
    }
    if (nouveau_handicap)
        init_aires_de_jeu();

    dessine_image_jeu(VSCR_VSCREEN, VSCR_AUTRES+1, VSCR_AUTRES+2, 1/*!fast_preview*/, 0);

    pv_larg_aire = ((x_fin[0]-x_deb[0])*2)/5;
    pv_haut_aire = ((y_fin[0]-y_deb[0])*2)/5;

    for (i = 0; i < nbj; i++)
    {
        pv_x1[i] = 96+(x_deb[i]*2)/5;
        pv_y1[i] = 60+(y_deb[i]*2)/5;
    }

    pv_tc_piece = (tc*2)/5;
    if (get_game_team())
    {
        i = nbj-1;
        while ( (pv_x1[i]+((pv_larg_aire-(4*pv_tc_piece))*(piece[i].p_x-x_min[i]))/(nbx-5)+(4*pv_tc_piece) >= pv_x1[0]+pv_larg_aire)
            && (pv_tc_piece > 1) )
            pv_tc_piece--;
    }
    if (pv_tc_piece == 0) pv_tc_piece = 1;

    pv_tc_next = (tc_next*2)/4;
    if (pv_tc_next == 0) pv_tc_next = 1;
    pv_tc_next2 = (pv_tc_next > TC_MIN)?pv_tc_next-1:TC_MIN;


    /* zoom pour mettre l'image en petit au milieu: */
    zoom(VSCR_VSCREEN, scr_nbx, scr_nby, VSCR_MENU, 128, 80, 96, 60);

    /* dessin des rectangles dans la petite image du milieu               */
    /* (pour eviter les lignes qui ne passent pas lors du retrecissement) */
    if (get_game_team())
    {
        switch (view_next)
        {
            case 1:
                while ( (pv_y1[0]-pv_tc_next*4+2 < 0)
                    && (pv_tc_next > TC_MIN) )
                    pv_tc_next--;
                pv_tc_next2 = (pv_tc_next > TC_MIN)?pv_tc_next-1:TC_MIN;
                break;
            case 2:
                while ( (pv_y1[0]-pv_tc_next*4+2-pv_tc_next2*4+2 < 0)
                    && (pv_tc_next > TC_MIN) )
                {
                    pv_tc_next--;
                    pv_tc_next2 = (pv_tc_next > TC_MIN)?pv_tc_next-1:TC_MIN;
                }
                break;
        }
        rectangle(VSCR_MENU, pv_x1[0], pv_y1[0],
            pv_x1[0]+pv_larg_aire, pv_y1[0]+pv_haut_aire,
            couleur_cadre[2]);
        if (get_window_msg())
            rectangle(VSCR_MENU, pv_x1[0], pv_y1[0]+pv_haut_aire+2,
                pv_x1[0]+pv_larg_aire, pv_y1[0]+pv_haut_aire+2+5,
                couleur_cadre[2]);
        for (i = 0; i < nbj; i++)
        {
            if (view_next > 0)
            {
                rectangle(VSCR_MENU, pv_x1[i]+((x_next[i]-x_deb[team[i]])*2)/5,
                    pv_y1[i]-pv_tc_next*4-2,
                    pv_x1[i]+((x_next[i]-x_deb[team[i]])*2)/5+4*pv_tc_next+1,
                    pv_y1[i]-1, couleur_cadre[2]);
                if (view_next == 2)
                    rectangle(VSCR_MENU, pv_x1[i]+((x_next2[i]-x_deb[team[i]])*2)/5,
                        pv_y1[i]-pv_tc_next*4-2-pv_tc_next2*4-2,
                        pv_x1[i]+((x_next2[i]-x_deb[team[i]])*2)/5+4*pv_tc_next2+1,
                        pv_y1[i]-pv_tc_next*4-2-1, couleur_cadre[2]);
            }
            if ( (i == 0) || (team[i] != team[i-1]) )
                rectangle(VSCR_MENU,
                    pv_x1[i]+((x_fin[team[i]]+2-x_deb[i])*2)/5,
                    pv_y1[i]+((y_carres_lignes[i][0]+7-y_deb[i])*2)/5,
                    pv_x1[i]+((x_fin[team[i]]+23-x_deb[i])*2)/5,
                    pv_y1[i]+((y_carres_lignes[i][0]+16-y_deb[i])*2)/5,
                    couleur_cadre[2]);
        }
        /*  attention si plusieurs joueurs ! */
        if (get_time_limite() > 0)
            rectangle(VSCR_MENU,
                pv_x1[0]+((x_time-2-x_deb[0])*2)/5,
                pv_y1[0]+((y_time-2-y_deb[0])*2)/5,
                pv_x1[0]+((x_time+19-x_deb[0])*2)/5,
                pv_y1[0]+((y_time-2-y_deb[0])*2)/5+4,     /* normalement y_time+7 mais pb d'arrondi */
                couleur_cadre[2]);
    }
    else
    {
        if ( (view_next > 0) && (nbj > 1) )
            while ( (pv_x1[0]+pv_larg_aire+1+pv_tc_next*4+1 >= pv_x1[1])
                && (pv_tc_next > TC_MIN) )
                pv_tc_next--;
        pv_tc_next2 = (pv_tc_next > TC_MIN)?pv_tc_next-1:TC_MIN;
        for (i = 0; i < nbj; i++)
        {
            rectangle(VSCR_MENU, pv_x1[i], pv_y1[i],
                pv_x1[i]+pv_larg_aire, pv_y1[i]+pv_haut_aire,
                couleur_cadre[ordi[i]]);
            if (get_window_msg())
                rectangle(VSCR_MENU, pv_x1[i], pv_y1[i]+pv_haut_aire+2,
                    pv_x1[i]+pv_larg_aire, pv_y1[i]+pv_haut_aire+2+5,
                    couleur_cadre[ordi[i]]);
            if (view_next > 0)
            {
                rectangle(VSCR_MENU, pv_x1[i]+pv_larg_aire+1, pv_y1[i],
                    /*pv_x1[i]+pv_larg_aire+((4*tc_next+5)*2)/5,*/
                    pv_x1[i]+pv_larg_aire+1+pv_tc_next*4+1,
                    /*pv_y1[i]+((4*tc_next+5)*2)/5,*/
                    pv_y1[i]+pv_tc_next*4+1,
                    couleur_cadre[ordi[i]]);
                if (view_next == 2)
                    rectangle(VSCR_MENU, pv_x1[i]+pv_larg_aire+1,
                        pv_y1[i]+pv_tc_next*4+3,
                        pv_x1[i]+pv_larg_aire+1+pv_tc_next2*4+1,
                        pv_y1[i]+pv_tc_next*4+3+pv_tc_next2*4+1,
                        couleur_cadre[ordi[i]]);
            }
            rectangle(VSCR_MENU,
                pv_x1[i]+((x_fin[team[i]]+2-x_deb[i])*2)/5,
                pv_y1[i]+((y_carres_lignes[i][0]+7-y_deb[i])*2)/5,
                pv_x1[i]+((x_fin[team[i]]+23-x_deb[i])*2)/5,
                pv_y1[i]+((y_carres_lignes[i][0]+16-y_deb[i])*2)/5,
                couleur_cadre[ordi[i]]);
            /*affiche_piece_preview(VSCR_MENU, i, &piece[i]);*/
            write_xy_s_c(VSCR_MENU, pv_x1[i]+1, pv_y1[i]+pv_haut_aire-5,
                28, -1, "%d", i+1);
        }
        if (get_time_limite() > 0)
            rectangle(VSCR_MENU,
                pv_x1[0]+((x_time-2-x_deb[0])*2)/5,
                pv_y1[0]+((y_time-2-y_deb[0])*2)/5,
                pv_x1[0]+((x_time+19-x_deb[0])*2)/5,
                pv_y1[0]+((y_time-2-y_deb[0])*2)/5+3,     /* normalement y_time+7 mais pb d'arrondi */
                couleur_cadre[0]);
    }

    /* affichage proprement dit ! */
    copy_v2s_xy(VSCR_MENU, 96, 60, 128, 80);

    affiche_pieces_suivantes_preview();

    /* force l'affichage des pieces des le 1er passage dans la fonction */
    /* 'descend_pieces_preview'                                         */
    if (nouvelles_pieces)
        pv_cptr_tombe = 1;
}

/*
 * descend_pieces_preview: fait descendre les pieces de la preview
 */

void descend_pieces_preview()
{
    int     i, retourne;

    pv_cptr_tombe--;
    if (pv_cptr_tombe == 0)
    {
        for (i = 0; i < nbj; i++)
        {
            efface_piece_preview(ECRAN_REEL, i, &piece[i]);
            if (peut_descendre(i, &piece[i]) == 0)
                piece[i].p_y++;
            else
                nouvelle_piece(i, 0);
            if (view_next > 0)
            {
                affiche_piece_suivante_preview(ECRAN_REEL, i, &next_piece[i]);
                if (view_next == 2)
                    affiche_piece_suivante2_preview(ECRAN_REEL, i, &next_piece2[i]);
            }
        }
        /*if ( (same_pieces) && (rand()%40 == 39) )
            for (i = 0; i < nbj; i++)
                retourne_piece(i, &piece[i]);*/
        if (same_pieces)
        {
            retourne = (rand()%35 == 34);
            for (i = 0; i < nbj; i++)
            {
                if (retourne)
                    retourne_piece(i, &piece[i]);
                affiche_piece_preview(ECRAN_REEL, i, &piece[i]);
            }
        }
        else
        {
            for (i = 0; i < nbj; i++)
            {
                /* 1 chance sur 35 de retourner la piece */
                if ( /*(!same_pieces) &&*/ (rand()%35 == 34) )
                    retourne_piece(i, &piece[i]);
                affiche_piece_preview(ECRAN_REEL, i, &piece[i]);
            }
        }
        pv_cptr_tombe = 5;
    }
}

/* ========================================================================== */

/*
 * aide_flashtris: affiche l'aide generale sur FlashTris
 */

void aide_flashtris()
{
    affiche_option(ECRAN_REEL, pos_menu, pos_advanced, pos_barre, 1, 0, 2);

    /* recuperation fond jeu au milieu */
    zoom(VSCR_AUTRES+1, scr_nbx, scr_nby, VSCR_VSCREEN, 128, 80, 96, 60);

    /* dessin help */
    if (langue == ANGLAIS)
    {
        write_xy_c_o(VSCR_VSCREEN, 98, 60, COUL_FLAMES, 5, 27, "FlashTris Help");

        write_xy_s_c(VSCR_VSCREEN, 98, 69, 50, 1, "MENU:");
        write_xy_s_c(VSCR_VSCREEN, 98, 75, 66, 2, "[Tab]");
        write_xy_s_c(VSCR_VSCREEN, 120, 75, 17, 2, "Next");
        write_xy_s_c(VSCR_VSCREEN, 106, 81, 66, 2, "[A]");
        write_xy_s_c(VSCR_VSCREEN, 120, 81, 17, 2, "Adv.");
        write_xy_s_c(VSCR_VSCREEN, 98, 87, 66, 2, "[Esc]");
        write_xy_s_c(VSCR_VSCREEN, 120, 87, 17, 2, "Quit");
        write_xy_s_c(VSCR_VSCREEN, 98, 93, 17, 2, "(+Ctrl=fast)");

        write_xy_s_c(VSCR_VSCREEN, 145, 69, 50, 1, "CHOICE:");
        write_xy_s_c(VSCR_VSCREEN, 145, 75, 66, 2, "Arrows,Home,");
        write_xy_s_c(VSCR_VSCREEN, 145, 81, 66, 2, "PgUp,Pgdn,");
        write_xy_s_c(VSCR_VSCREEN, 189, 81, 17, 2, "Move bar");
        write_xy_s_c(VSCR_VSCREEN, 145, 87, 66, 2, "[Enter]");
        write_xy_s_c(VSCR_VSCREEN, 176, 87, 17, 2, "Select");
        write_xy_s_c(VSCR_VSCREEN, 145, 93, 66, 2, "[Space]");
        write_xy_s_c(VSCR_VSCREEN, 176, 93, 17, 2, "(un)check");
        write_xy_s_c(VSCR_VSCREEN, 145, 99, 66, 2, "[F1/F2]");
        write_xy_s_c(VSCR_VSCREEN, 176, 99, 17, 2, "local help");

        write_xy_s_c(VSCR_VSCREEN, 98, 99, 50, 1, "SOUND:");
        write_xy_s_c(VSCR_VSCREEN, 106, 105, 66, 2, "[+/-]");
        write_xy_s_c(VSCR_VSCREEN, 127, 105, 17, 2, "Volume (+Ctrl=Fast)");
        write_xy_s_c(VSCR_VSCREEN, 98, 111, 66, 2, "[F5/F6]");
        write_xy_s_c(VSCR_VSCREEN, 127, 111, 17, 2, "Prev./next music");
        write_xy_s_c(VSCR_VSCREEN, 108, 117, 66, 2, "[F11]");
        write_xy_s_c(VSCR_VSCREEN, 127, 117, 17, 2, "Music: previous pattern");
        write_xy_s_c(VSCR_VSCREEN, 107, 123, 66, 2, "[F12]");
        write_xy_s_c(VSCR_VSCREEN, 127, 123, 17, 2, "Music: next pattern");
        write_xy_s_c(VSCR_VSCREEN, 114, 129, 66, 2, "[R]");
        write_xy_s_c(VSCR_VSCREEN, 127, 129, 17, 2, "Random music");

        write_xy_s_c(VSCR_VSCREEN, 142, 135, COUL_FLAMES+5*6, -6, "Press Esc");
    }
    else
    {
        write_xy_c_o(VSCR_VSCREEN, 98, 60, COUL_FLAMES, 5, 27, "Aide FlashTris");

        write_xy_s_c(VSCR_VSCREEN, 98, 69, 50, 1, "MENU:");
        write_xy_s_c(VSCR_VSCREEN, 98, 75, 66, 2, "[Tab]");
        write_xy_s_c(VSCR_VSCREEN, 118, 75, 17, 2, "Suiv.");
        write_xy_s_c(VSCR_VSCREEN, 106, 81, 66, 2, "[A]");
        write_xy_s_c(VSCR_VSCREEN, 118, 81, 17, 2, "Avanc.");
        write_xy_s_c(VSCR_VSCREEN, 98, 87, 66, 2, "[Esc]");
        write_xy_s_c(VSCR_VSCREEN, 118, 87, 17, 2, "Quitte");
        write_xy_s_c(VSCR_VSCREEN, 98, 93, 17, 2, "(+Ctrl=rap.)");

        write_xy_s_c(VSCR_VSCREEN, 147, 69, 50, 1, "CHOIX:");
        write_xy_s_c(VSCR_VSCREEN, 147, 75, 66, 2, "Fleches,Home,");
        write_xy_s_c(VSCR_VSCREEN, 147, 81, 66, 2, "PgUp,Pgdn,");
        write_xy_s_c(VSCR_VSCREEN, 201, 75, 17, 2, "Bouge");
        write_xy_s_c(VSCR_VSCREEN, 201, 81, 17, 2, "barre");
        write_xy_s_c(VSCR_VSCREEN, 147, 87, 66, 2, "[Entree]");
        write_xy_s_c(VSCR_VSCREEN, 180, 87, 17, 2, "Selection");
        write_xy_s_c(VSCR_VSCREEN, 147, 93, 66, 2, "[Espace]");
        write_xy_s_c(VSCR_VSCREEN, 180, 93, 17, 2, "(de)coche");
        write_xy_s_c(VSCR_VSCREEN, 147, 99, 66, 2, "[F1/F2]");
        write_xy_s_c(VSCR_VSCREEN, 180, 99, 17, 2, "aide locale");

        write_xy_s_c(VSCR_VSCREEN, 98, 99, 50, 1, "SON:");
        write_xy_s_c(VSCR_VSCREEN, 106, 105, 66, 2, "[+/-]");
        write_xy_s_c(VSCR_VSCREEN, 127, 105, 17, 2, "Volume (+Ctrl=Rapide)");
        write_xy_s_c(VSCR_VSCREEN, 98, 111, 66, 2, "[F5/F6]");
        write_xy_s_c(VSCR_VSCREEN, 127, 111, 17, 2, "Musik prec./suiv.");
        write_xy_s_c(VSCR_VSCREEN, 108, 117, 66, 2, "[F11]");
        write_xy_s_c(VSCR_VSCREEN, 127, 117, 17, 2, "Musik: pattern preced.");
        write_xy_s_c(VSCR_VSCREEN, 107, 123, 66, 2, "[F12]");
        write_xy_s_c(VSCR_VSCREEN, 127, 123, 17, 2, "Musik: pattern suivant");
        write_xy_s_c(VSCR_VSCREEN, 114, 129, 66, 2, "[R]");
        write_xy_s_c(VSCR_VSCREEN, 127, 129, 17, 2, "Musik aleatoire");

        write_xy_s_c(VSCR_VSCREEN, 126, 135, COUL_FLAMES+5*6, -6, "Appuyez sur Echap");
    }

    /* recopie dans ecran reel */
    copy_v2s_xy(VSCR_VSCREEN, 96, 60, 128, 80);

    aide_affichee = 1;
}

/*
 * quitte_aide_flashtris: quitte l'aide de FlashTris
 */

void quitte_aide_flashtris()
{
    int     i;

    if ( (sb16) && (sons_menu) )
        SON_BOUGE_BARRE;
    redessine_preview(0, 0);
    for (i = 0; i < nbj; i++)
        affiche_piece_preview(ECRAN_REEL, i, &piece[i]);
    affiche_option(ECRAN_REEL, pos_menu, pos_advanced, pos_barre, 1, 1, 0);
    aide_affichee = 0;
}

/* ========================================================================== */

int             p_edit_x[4]    = { 100, 148, 173, 198 };
int             p_edit_y[4]    = { 82, 90, 98, 106 };
int             p_edit_tc_x[4] = { 10, 4, 4, 4 };
int             p_edit_tc_y[4] = { 9, 3, 3, 3 };
unsigned char   *texte_piece_editor[2][4] =
                { { "Default", "R+1", "R+2", "R+3" },
                  { "Defaut", "R+1", "R+2", "R+3" } };

/*
 * pieces_editor_affiche_case: affiche une case de l'editeur de pieces
 *                             Si contenu = 1 on affiche le contenu
 *                             Si bordure = 1 on affiche le bord non actif
 *                                          2 on affiche le bord actif (violet)
 */

void pieces_editor_affiche_case(int num_piece, int position, int contenu,
                                int bordure)
{
    int     i, x, y, nbx, nby;

    x = position%4;
    y = position/4;

    if (contenu)
    {
        if ((editor_piece[num_piece] & (1 << (15-position))) != 0)
        {
            for (i = 0; i < p_edit_tc_y[num_piece]; i++)
                ligne_horiz(ECRAN_REEL,
                    p_edit_x[num_piece]+1+x*(p_edit_tc_x[num_piece]+1),
                    p_edit_x[num_piece]+1+x*(p_edit_tc_x[num_piece]+1)+p_edit_tc_x[num_piece]-1,
                    i+p_edit_y[num_piece]+1+y*(p_edit_tc_y[num_piece]+1),
                    96+i*(3-(p_edit_tc_y[num_piece]/4)));
        }
        else
            copy_v2s_xy(VSCR_VSCREEN,
                p_edit_x[num_piece]+1+x*(p_edit_tc_x[num_piece]+1),
                p_edit_y[num_piece]+1+y*(p_edit_tc_y[num_piece]+1),
                p_edit_tc_x[num_piece], p_edit_tc_y[num_piece]);
    }

    if (bordure == 2)
    {
        nbx = (p_edit_x[num_piece]+1+x*(p_edit_tc_x[num_piece]+1)+p_edit_tc_x[num_piece]-1) -
                (p_edit_x[num_piece]+1+x*(p_edit_tc_x[num_piece]+1));
        nby = (p_edit_y[num_piece]+1+y*(p_edit_tc_y[num_piece]+1)+p_edit_tc_y[num_piece]-1) -
                (p_edit_y[num_piece]+1+y*(p_edit_tc_y[num_piece]+1));
        dessin_cadre_option(p_edit_x[num_piece]+1+x*(p_edit_tc_x[num_piece]+1)+1,
            p_edit_y[num_piece]+y*(p_edit_tc_y[num_piece]+1)+2, nbx-2, nby-2);
    }
    else
        if (bordure == 1)
        {
            ligne_horiz(ECRAN_REEL, p_edit_x[num_piece]+1+x*(p_edit_tc_x[num_piece]+1),
                p_edit_x[num_piece]+1+x*(p_edit_tc_x[num_piece]+1)+p_edit_tc_x[num_piece]-1,
                p_edit_y[num_piece]+y*(p_edit_tc_y[num_piece]+1), 5);
            ligne_horiz(ECRAN_REEL, p_edit_x[num_piece]+1+x*(p_edit_tc_x[num_piece]+1),
                p_edit_x[num_piece]+1+x*(p_edit_tc_x[num_piece]+1)+p_edit_tc_x[num_piece]-1,
                p_edit_y[num_piece]+(y+1)*(p_edit_tc_y[num_piece]+1), 5);
            ligne_vertic(ECRAN_REEL, p_edit_x[num_piece]+x*(p_edit_tc_x[num_piece]+1),
                p_edit_y[num_piece]+1+y*(p_edit_tc_y[num_piece]+1),
                p_edit_y[num_piece]+1+y*(p_edit_tc_y[num_piece]+1)+p_edit_tc_y[num_piece]-1, 5);
            ligne_vertic(ECRAN_REEL, p_edit_x[num_piece]+(x+1)*(p_edit_tc_x[num_piece]+1),
                p_edit_y[num_piece]+1+y*(p_edit_tc_y[num_piece]+1),
                p_edit_y[num_piece]+1+y*(p_edit_tc_y[num_piece]+1)+p_edit_tc_y[num_piece]-1, 5);
        }
}

/*
 * pieces_editor_retourne_piece: retourne la piece editee
 */

void pieces_editor_retourne_piece(int piece, int *rotation1, int *rotation2,
                                  int *rotation3)
{
    *rotation1 = ((piece & 8) >> 3)+            /* rangee du bas */
                 ((piece & 128) >> 6)+
                 ((piece & 2048) >> 9)+
                 ((piece & 32768) >> 12)+
                 ((piece & 4) << 2)+            /* rangee milieu bas */
                 ((piece & 64) >> 1)+
                 ((piece & 1024) >> 4)+
                 ((piece & 16384) >> 7)+
                 ((piece & 2) << 7)+            /* rangee milieu haut */
                 ((piece & 32) << 4)+
                 ((piece & 512) << 1)+
                 ((piece & 8192) >> 2)+
                 ((piece & 1) << 12)+           /* rangee du haut */
                 ((piece & 16) << 9)+
                 ((piece & 256) << 6)+
                 ((piece & 4096) << 3);

    *rotation2 = (((*rotation1) & 8) >> 3)+     /* rangee du bas */
                 (((*rotation1) & 128) >> 6)+
                 (((*rotation1) & 2048) >> 9)+
                 (((*rotation1) & 32768) >> 12)+
                 (((*rotation1) & 4) << 2)+     /* rangee milieu bas */
                 (((*rotation1) & 64) >> 1)+
                 (((*rotation1) & 1024) >> 4)+
                 (((*rotation1) & 16384) >> 7)+
                 (((*rotation1) & 2) << 7)+     /* rangee milieu haut */
                 (((*rotation1) & 32) << 4)+
                 (((*rotation1) & 512) << 1)+
                 (((*rotation1) & 8192) >> 2)+
                 (((*rotation1) & 1) << 12)+    /* rangee du haut */
                 (((*rotation1) & 16) << 9)+
                 (((*rotation1) & 256) << 6)+
                 (((*rotation1) & 4096) << 3);

    *rotation3 = (((*rotation2) & 8) >> 3)+     /* rangee du bas */
                 (((*rotation2) & 128) >> 6)+
                 (((*rotation2) & 2048) >> 9)+
                 (((*rotation2) & 32768) >> 12)+
                 (((*rotation2) & 4) << 2)+     /* rangee milieu bas */
                 (((*rotation2) & 64) >> 1)+
                 (((*rotation2) & 1024) >> 4)+
                 (((*rotation2) & 16384) >> 7)+
                 (((*rotation2) & 2) << 7)+     /* rangee milieu haut */
                 (((*rotation2) & 32) << 4)+
                 (((*rotation2) & 512) << 1)+
                 (((*rotation2) & 8192) >> 2)+
                 (((*rotation2) & 1) << 12)+    /* rangee du haut */
                 (((*rotation2) & 16) << 9)+
                 (((*rotation2) & 256) << 6)+
                 (((*rotation2) & 4096) << 3);
}

/*
 * prepare_pieces_editor: prepare l'editeur de pieces
 */

void prepare_pieces_editor()
{
    int     num_piece, i, j;

    affiche_option(ECRAN_REEL, pos_menu, pos_advanced, pos_barre, 1, 0, 2);

    /* recuperation fond jeu au milieu */
    zoom(VSCR_AUTRES+1, scr_nbx, scr_nby, VSCR_VSCREEN, 128, 80, 96, 60);

    /* dessin editeur de pieces */
    if (langue == ANGLAIS)
    {
        write_xy_c_o(VSCR_VSCREEN, 104, 61, COUL_FLAMES, 5,/*78, -2,*/ 27, "Pieces");
        write_xy_c_o(VSCR_VSCREEN, 164, 61, COUL_FLAMES, 5, 27, "editor");
        write_xy_s_c(VSCR_VSCREEN, 98, 134, 18, 2, "[Arrows,Space,Ins,Del,Tab]");
    }
    else
    {
        write_xy_c_o(VSCR_VSCREEN, 99, 61, COUL_FLAMES, 5, 27, "Editeur");
        write_xy_c_o(VSCR_VSCREEN, 168, 61, COUL_FLAMES, 5, 27, "pieces");
        write_xy_s_c(VSCR_VSCREEN, 98, 134, 18, 2, "[Fleches,Espace,Ins,Del,Tab]");
    }

    for (num_piece = 0; num_piece < 4; num_piece++)
    {
        write_xy_s_c(VSCR_VSCREEN, p_edit_x[num_piece], p_edit_y[num_piece]-6,
            116, 2, texte_piece_editor[langue][num_piece]);
        for (i = p_edit_x[num_piece]+1; i < p_edit_x[num_piece]+4*(p_edit_tc_x[num_piece]+1); i++)
            for (j = p_edit_y[num_piece]+1; j < p_edit_y[num_piece]+4*(p_edit_tc_y[num_piece]+1); j++)
                assombrit_pixel(VSCR_VSCREEN, VSCR_VSCREEN, i, j, 31, 31, 9);
        for (i = 0; i < 4; i++)
            for (j = 0; j < 5; j++)
            {
                ligne_horiz(VSCR_VSCREEN, p_edit_x[num_piece]+1+i*(p_edit_tc_x[num_piece]+1),
                    p_edit_x[num_piece]+1+i*(p_edit_tc_x[num_piece]+1)+p_edit_tc_x[num_piece]-1,
                    p_edit_y[num_piece]+j*(p_edit_tc_y[num_piece]+1), 5);
                ligne_vertic(VSCR_VSCREEN, p_edit_x[num_piece]+j*(p_edit_tc_x[num_piece]+1),
                    p_edit_y[num_piece]+1+i*(p_edit_tc_y[num_piece]+1),
                    p_edit_y[num_piece]+1+i*(p_edit_tc_y[num_piece]+1)+p_edit_tc_y[num_piece]-1, 5);
            }
    }

    /* recopie dans ecran reel */
    copy_v2s_xy(VSCR_VSCREEN, 96, 60, 128, 80);

    editor_pos = 0;
    editor_piece[0] = 0;
    editor_piece[1] = 0;
    editor_piece[2] = 0;
    editor_piece[3] = 0;

    pieces_editor_affiche_case(0, editor_pos, 0, 2);

    pieces_editor = 1;
}

/*
 * quitte_pieces_editor: quitte l'editeur de pieces
 */

void quitte_pieces_editor()
{
    int     i;

    if ( (sb16) && (sons_menu) )
        SON_BOUGE_BARRE;
    redessine_preview(0, 0);
    for (i = 0; i < nbj; i++)
        affiche_piece_preview(ECRAN_REEL, i, &piece[i]);
    affiche_option(ECRAN_REEL, pos_menu, pos_advanced, pos_barre, 1, 1, 0);
    pieces_editor = 0;
}

/* ========================================================================== */

/*
 * prepare_redefine_keys: prepare la redefinition des touches
 */

void prepare_redefine_keys()
{
    affiche_option(ECRAN_REEL, pos_menu, pos_advanced, pos_barre, 1, 0, 2);

    /* recuperation fond jeu au milieu */
    zoom(VSCR_AUTRES+1, scr_nbx, scr_nby, VSCR_VSCREEN, 128, 80, 96, 60);

    /* dessin redefine keys */
    if (langue == ANGLAIS)
    {
        write_xy_c_o(VSCR_VSCREEN, 142, 61, COUL_FLAMES, 5, 27, "Keys");
        write_xy_s_c(VSCR_VSCREEN, 98, 134, 18, 2, "[Arrows,Enter]");
        write_xy_s_c_o(VSCR_VSCREEN, 126, 94, 49, 1, 4, "Not implemented !");
    }
    else
    {
        write_xy_c_o(VSCR_VSCREEN, 129, 61, COUL_FLAMES, 5, 27, "Touches");
        write_xy_s_c(VSCR_VSCREEN, 98, 134, 18, 2, "[Fleches,Entree]");
        write_xy_s_c_o(VSCR_VSCREEN, 130, 94, 49, 1, 4, "Non developpe !");
    }

    /* recopie dans ecran reel */
    copy_v2s_xy(VSCR_VSCREEN, 96, 60, 128, 80);

    redefine_keys = 1;
}

void quitte_redefine_keys()
{
    int     i;

    if ( (sb16) && (sons_menu) )
        SON_BOUGE_BARRE;
    redessine_preview(0, 0);
    for (i = 0; i < nbj; i++)
        affiche_piece_preview(ECRAN_REEL, i, &piece[i]);
    affiche_option(ECRAN_REEL, pos_menu, pos_advanced, pos_barre, 1, 1, 0);
    redefine_keys = 0;
}

/* ========================================================================== */

void rotation_palette_barre()
{
    unsigned char   pal_temp[3];
    int             i;

    compteur_palette++;
    if (compteur_palette == 1)
    {
        compteur_palette = 0;
        memcpy(pal_temp, palette+160*3, 3);
        memmove(palette+160*3, palette+161*3, 14*3);
        memcpy(palette+174*3, pal_temp, 3);
        for (i = 160; i <= 174; i++)
            setcoul(i, palette[i*3], palette[i*3+1], palette[i*3+2]);
    }
}

/*void affiche_vu_metres()
{
    int     i, j, hauteur, x, somme;

    for (i = 0; i < 16; i++)
    {
        x = X_VU_METRES+i*2;
        somme = 0;
        for (j = i*32; j <= i*32+31; j++)
        {
            hauteur = (int)((signed char)dmabuffer[j]);
            if (hauteur < 0) hauteur = -hauteur;
            hauteur = hauteur/16;
            somme += hauteur;
        }
        somme = somme/32;
        //printf("hauteur %d: %d\n", i, dmabuffer[i*32]);
        for (j = 0; j < somme; j++)
            putpixel_ofs(x+(Y_VU_METRES-j)*scr_nbx, 176+(j*8));
        for (j = somme; j < 8; j++)
            putpixel_ofs(x+(Y_VU_METRES-j)*scr_nbx, vscr_menu[x+(Y_VU_METRES-j)*scr_nbx]);
    }

    int     i, j, l, m, y, x, nb_colonnes, intervalle, nombre;

    nb_colonnes = 8;
    intervalle = bpmsamples/nb_colonnes;
    x = X_VU_METRES;
    for (i = 0; i < numtracks; i++)
    {
        l = 0;
        for (j = 0; j < nb_colonnes; j++)
        {
            nombre = 0;
            for (m = 0; m < intervalle; m++)
            {
                nombre += buffer_pistes[i*1000+l];
                l++;
            }
            nombre = nombre/intervalle;
            if (nombre >= 0x80) nombre -= 0x80;
            nombre = nombre/16;
            for (y = 0; y < nombre; y++)
                putpixel_ofs(x+(Y_VU_METRES-y)*scr_nbx, 176+(y*8));
            for (y = nombre; y < 8; y++)
                putpixel_ofs(x+(Y_VU_METRES-y)*scr_nbx, vscr_menu[x+(Y_VU_METRES-y)*scr_nbx]);
            x += 2;
        }
        x += 8;
    }
}*/

#define X_INSTRUMENT    120
#define Y_INSTRUMENT    170

int     instrument[8] = { -1, -1, -1, -1, -1, -1, -1, -1 };

void affiche_instrument(int voix, int position, int total)
{
    int     x, y, coul, longueur;

    if (instrument[voix] != -1)
    {
        x = (instrument[voix]/4);
        y = (instrument[voix]%4);
        if (total == 0)
            coul = COUL_FLAMES+63;
        else
        {
            if ((total%2) != 0) total++;
            if (position >= (total/2)) position = total-position;
            coul = COUL_FLAMES+(position*63)/(total/2);
        }
        longueur = (coul-COUL_FLAMES)/8+1;
        ligne_horiz(ECRAN_REEL, X_INSTRUMENT+x*10, X_INSTRUMENT+x*10+longueur-1, Y_INSTRUMENT+y*2, coul);
        copy_v2s_xy(VSCR_MENU, X_INSTRUMENT+x*10+longueur, Y_INSTRUMENT+y*2, 8-longueur, 1);
    }
}

void efface_instrument(int voix)
{
    int     x, y;

    if (instrument[voix] != -1)
    {
        x = (instrument[voix]/4);
        y = (instrument[voix]%4);
        /*ligne_horiz(X_INSTRUMENT+x*6, X_INSTRUMENT+x*6+3, Y_INSTRUMENT+y*2, 23);*/
        copy_v2s_xy(VSCR_MENU, X_INSTRUMENT+x*10, Y_INSTRUMENT+y*2, 8, 1);
        instrument[voix] = -1;
    }
}

void affiche_instruments()
{
    int     position, instr, i, pos, len;

    if (row != 65)
    {
        position = (order[orderpos-1]*size_pattern)+((row-1)*numtracks*4);
        for (i = 0; i < numtracks; i++)
        {
            instr = (patterns[position+i*4] & 0xF0) | (patterns[position+i*4+2] >> 4);
            pos = tracks[i].position;
            len = tracks[i].len;
            if ( (instr != 0) || ( (pos > 0) && (pos < len) ) )
            {
                if (instr != 0)
                {
                    instr--;
                    if ( (instr != instrument[i]) && (instrument[i] != -1) )
                        efface_instrument(i);
                    instrument[i] = instr;
                    if ( (pos > 0) && (pos < len) )
                    {

                        affiche_instrument(i, tracks[i].samples[pos], 256);
                        /*affiche_instrument(i, 100, 255);*/
                    }
                }
            }
            else
                efface_instrument(i);
        }
    }
}

/*
 * affiche_barres_samples: affiche des barres pour les samples
 *                         Chaque barre indique la progression dans un sample
 */

/*void affiche_barres_samples()
{
    int     i, j, hauteur;

    for (i = 0; i < 4; i++)
    {
        if ( (tracks[i].samples == NULL)
            || (tracks[i].len == 0)
            || (tracks[i].position == 0)
            || (tracks[i].position >= tracks[i].len) )
            hauteur = 0;
        else
            hauteur = (tracks[i].position*barsmp_haut[barres_samples-1])/tracks[i].len;
        for (j = 0; j <= hauteur; j++)
            ligne_horiz(ECRAN_REEL,
                X_VOLUME+barsmp_x[barres_samples-1][i],
                X_VOLUME+barsmp_x[barres_samples-1][i]+barsmp_larg[barres_samples-1]-1,
                Y_VOLUME-1-j,
                COUL_FLAMES+j*barsmp_coul[barres_samples-1]);
        copy_v2s_xy(VSCR_MENU,
            X_VOLUME+barsmp_x[barres_samples-1][i],
            Y_VOLUME-1-barsmp_haut[barres_samples-1]-1,
            barsmp_larg[barres_samples-1],
            barsmp_haut[barres_samples-1]+1-hauteur);
    }
}*/

/*
 * affiche_barres_samples: affiche des barres pour les samples
 *                         Chaque barre indique la puissance de la note jouee
 */

void affiche_barres_samples()
{
    int     i, j, hauteur;

    if (!module_fini)
    {
        for (i = 0; i < 4; i++)
        {
            if ( (tracks[i].samples == NULL)
                || (tracks[i].len == 0)
                || (tracks[i].position == 0)
                || (tracks[i].position >= tracks[i].len) )
                hauteur = 0;
            else
            {
                hauteur = tracks[i].samples[tracks[i].position];
                if (hauteur >= 128)
                    hauteur = hauteur-128;
                else
                    hauteur = 127-hauteur;
                hauteur = (hauteur*barsmp_haut[barres_samples-1])/127;
            }
            for (j = 0; j <= hauteur; j++)
                ligne_horiz(ECRAN_REEL,
                    X_VOLUME+barsmp_x[barres_samples-1][i],
                    X_VOLUME+barsmp_x[barres_samples-1][i]+barsmp_larg[barres_samples-1]-1,
                    Y_VOLUME-1-j,
                    COUL_FLAMES+j*barsmp_coul[barres_samples-1]);
            copy_v2s_xy(VSCR_MENU,
                X_VOLUME+barsmp_x[barres_samples-1][i],
                Y_VOLUME-1-barsmp_haut[barres_samples-1]-1,
                barsmp_larg[barres_samples-1],
                barsmp_haut[barres_samples-1]+1-hauteur);
        }
    }
}
