/* ====
   ==== FlashTris Game, (c) 1999-2001 by FlashCode (Sebastien Helleu)
   ====
   ====    web: http://flashcode.free.fr
   ==== e-mail: flashcode@free.fr
   ====
   ==== netftris.c: gestion du reseau dans FlashTris
   ====

   ==== This file is part of FlashTris.
   ====
   ==== FlashTris is free software; you can redistribute it and/or modify
   ==== it under the terms of the GNU General Public License as published by
   ==== the Free Software Foundation; either version 2 of the License, or
   ==== (at your option) any later version.
   ====
   ==== FlashTris is distributed in the hope that it will be useful,
   ==== but WITHOUT ANY WARRANTY; without even the implied warranty of
   ==== MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   ==== GNU General Public License for more details.
   ====
   ==== You should have received a copy of the GNU General Public License
   ==== along with FlashTris; if not, write to the Free Software
   ==== Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Auteur  : FlashCode <flashcode@free.fr>
Creation: 1999/06/21

2001/09/06: FlashCode:
    - ajout de la licence
*/


#include <stdio.h>

#include "ftris.h"
#include "netftris.h"
#include "net.h"
#include "menu.h"
#include "menu_gfx.h"
#include "game.h"
#include "key.h"


int             network;            /* = 1 si reseau active                           */
int             net_master;         /* = 1 si le PC est le maitre de la partie reseau */
int             net_type;           /* type de reseau (UDP, IPX, modem, Internet)     */
unsigned char   net_nom[11];        /* nom reseau                                     */
unsigned char   net_server[51];     /* nom du serveur (machine qui a cree la partie)  */
int             net_nbj_max;        /* nombre de joueurs maxi pour la partie reseau   */
int             net_allow_bots;     /* autorise les ordinateurs sur les autres PC     */
int             net_death;          /* chacun pour soi (y-compris sur le meme PC)     */
int             net_view_other;     /* voir les autres ?                              */
int             net_bonus_level;    /* niveaux bonus en reseau ?                      */
int             net_same_pieces;    /* memes pieces en reseau ?                       */
int             net_chat;           /* chat autorise ?                                */
int             net_port_serveur;   /* numero de port utilise                         */


unsigned long   net_adresse_serveur;    /* adresse du serveur (si PC client !)  */
unsigned long   net_adresse_locale;     /* adresse locale de la machine         */
int             net_port_local;         /* port local (pour recevoir)           */
                                        /* il est different du port du serveur  */
                                        /* si le serveur et le client sont sur  */
                                        /* le meme PC (il est alors trouve      */
                                        /* automatiquement par balayage, voir   */
                                        /* la fonction init)                    */

int             cptr_net_retry;         /* compteur pour reenvoyer une requete vers le serveur */

unsigned char   message_recu[MAX_LONG_MSG]; /* message en reception     */
int             long_message_recu;          /* longueur du message recu */
unsigned long   adresse_envoyeur;           /* adresse de l'envoyeur    */


t_net_driver    net_driver[NB_PROTO] =
                { /*   UDP  */
                  { udp_init,   udp_send,   udp_recv,   udp_close,
                    udp_nom2addr },
                  /*   IPX  */
                  { ipx_init,   ipx_send,   ipx_recv,   ipx_close,
                    ipx_nom2addr },
                  /*  modem */
                  { modem_init, modem_send, modem_recv, modem_close,
                    modem_nom2addr } };


/* FORMAT DES MESSAGES FLASHTRIS :

          Octet   Contenu

            0     Type d'envoyeur      (S = serveur , C = client, * = sans objet)
            1     Type de destinataire (S = serveur , C = client, * = peu importe)
           2-3    Port local du PC
            4     Type de message (voir netftris.h)
          5 et +  Corps du message
*/

/* MESSAGES DE FLASHTRIS:

                           SERVEUR   CLIENT

        CONNECT_REQUEST         <------     (menu)

        CONNECT_ACCEPTED        ------>     (menu)

        CONNECT_REFUSED         ------>     (menu)

        DISCONNECT              <----->     (menu/jeu)

        START_GAME              ------>     (menu)

        LIGNES                  <----->     (jeu)
*/

/* ========================================================================== */

/*
 * net_init: initialise le reseau
 *           Renvoie: 0 si OK
 *                    <0 si probleme d'initialisation du reseau (voir net.h)
 */

int net_init()
{
    int     i, ret;

    ret = net_driver[net_type].init(net_port_serveur, &net_adresse_locale);
    if (ret == -2)
    {
        /*if (langue == ANGLAIS)
            ajoute_texte_preview(-1, "Searching port, please wait...");
        else
            ajoute_texte_preview(-1, "Recherche port, patientez...");*/
        for (i = net_port_serveur+1; i <= net_port_serveur+100; i++)
        {
            ret = net_driver[net_type].init(i, &net_adresse_locale);
            if (ret != -2)
            {
                net_port_local = i;
                break;
            }
        }
        return ret;
    }
    else
    {
        net_port_local = net_port_serveur;
        return ret;
    }
}

/*
 * net_connect: envoie une requete de connexion vers le serveur
 *              Renvoie: 1 si OK (requete envoyee)
 *                       0 si probleme (adresse serveur non trouvee)
 */

int net_connect(unsigned char *nom_serveur)
{
    unsigned char   message[MAX_LONG_MSG];
    unsigned long   adr_serveur;

    adr_serveur = net_driver[net_type].nom2addr(nom_serveur);
    if (adr_serveur == 0)
    {
        if (langue == ANGLAIS)
            ajoute_texte_preview(-1, "Server not found !");
        else
            ajoute_texte_preview(-1, "Serveur non trouve !");
        cptr_net_retry = 0;
        return 0;
    }
    else
    {
        sprintf(message, "CS%c%c%c%c%c%c%s",
            (unsigned char)(net_port_local >> 8),
            (unsigned char)(net_port_local & 255),
            NETFTRIS_CONNECT_REQUEST,
            (unsigned char)(nbp+'0'), (unsigned char)(nbc+'0'),
            (unsigned char)(strlen(net_nom)), net_nom);
        net_driver[net_type].send(adr_serveur, net_port_serveur,
            8+strlen(net_nom), message);
        cptr_net_retry = DELAI_RETRY;
        return 1;
    }
}

/*
 * net_send: envoie un message par le reseau
 *           Renvoie: 1 si OK
 *                    0 si probleme d'envoi (voir net.h)
 */

int net_send(int vers_joueur, int longueur, unsigned char *message)
{
    return net_driver[net_type].send(PC_net[vers_joueur].adresse,
        PC_net[vers_joueur].port, longueur, message);
}

/*
 * net_recv: scrute les messages par le reseau
 *           Renvoie: 1: un message a ete recu
 *                    0: rien n'est recu (file d'attente de reception vide)
 */

int net_recv()
{
    return net_driver[net_type].recv(MAX_LONG_MSG, message_recu,
        &long_message_recu, &adresse_envoyeur);
}

/*
 * net_close: ferme le reseau
 *            Renvoie: 1 si OK
 *                     0 si probleme de fermeture (voir net.h)
 */

int net_close()
{
    return net_driver[net_type].close();
}

/*
 * traite_message_recu: traite un message recu par reseau dans FlashTris
 */

void traite_message_recu()
{
    unsigned char   message[MAX_LONG_MSG], chaine[100], nom_serveur[11];
    int             i, j, k, l, port_envoyeur;

    /* DEBUG : !!! */
    /*printf("message recu longueur=%d\n", long_message_recu);*/
    /*printf("msg recu de:%lu (%lu.%lu.%lu.%lu)  longueur=%d  contenu=",
        adresse_envoyeur,
        (adresse_envoyeur & 255),
        ((adresse_envoyeur >> 8) & 255),
        ((adresse_envoyeur >> 16) & 255),
        ((adresse_envoyeur >> 24) & 255),
        long_message_recu);*/
    /*for (i = 0; i < long_message_recu; i++)
        printf("%c", message_recu[i]);*/
    /*printf("\n");*/

    if ( (message_recu[1] == 'C') && (net_master) )
    {
        /* ignore car le message est destine a un client */
        /* et ce PC est en serveur                       */
        return;
    }
    if ( (message_recu[1] == 'S') && (!net_master) )
    {
        /* ignore car le message est destine a un serveur */
        /* et ce PC est en client                         */
        return;
    }

    port_envoyeur = ((int)(message_recu[2]) << 8)+(int)(message_recu[3]);
    switch (message_recu[4])
    {
        case NETFTRIS_CONNECT_REQUEST:
            if (net_master)
            {
                /* le joueur est deja connecte ? */
                for (i = 1; i < nb_PC_net; i++)
                    if ( (PC_net[i].adresse == adresse_envoyeur)
                        && (PC_net[i].port == port_envoyeur) )
                    return;
                if (nb_PC_net < net_nbj_max)
                {
                    PC_net[nb_PC_net].nbp = (int)(message_recu[5]-'0');
                    PC_net[nb_PC_net].nbc = (int)(message_recu[6]-'0');
                    if ( (PC_net[nb_PC_net].nbc > 0) && (!net_allow_bots) )
                    {
                        sprintf(message, "SC%c%c%c%c%c%s",
                            (unsigned char)(net_port_local >> 8),
                            (unsigned char)(net_port_local & 255),
                            NETFTRIS_CONNECT_REFUSED,
                            NETFTRIS_REFUS_BOTS,
                            (unsigned char)(strlen(net_nom)), net_nom);
                        net_driver[net_type].send(adresse_envoyeur, port_envoyeur,
                            7+strlen(net_nom), message);
                    }
                    else
                    {
                        PC_net[nb_PC_net].nbj =
                            PC_net[nb_PC_net].nbp+PC_net[nb_PC_net].nbc;
                        PC_net[nb_PC_net].nb_perdus = 0;
                        PC_net[nb_PC_net].adresse = adresse_envoyeur;
                        PC_net[nb_PC_net].port = port_envoyeur;
                        j = (int)(message_recu[7]);
                        for (i = 0; i < j; i++)
                            PC_net[nb_PC_net].net_nom[i] = message_recu[8+i];
                        PC_net[nb_PC_net].net_nom[j] = '\0';
                        nb_PC_net++;
                        sprintf(message, "SC%c%c%c%c%s",
                            (unsigned char)(net_port_local >> 8),
                            (unsigned char)(net_port_local & 255),
                            NETFTRIS_CONNECT_ACCEPTED,
                            (unsigned char)(strlen(net_nom)), net_nom);
                        net_driver[net_type].send(
                            PC_net[nb_PC_net-1].adresse,
                            PC_net[nb_PC_net-1].port, 6+strlen(net_nom), message);
                        if (nb_PC_net > 2)
                            retire_texte_preview_derniere_ligne();
                        if (langue == ANGLAIS)
                            sprintf(chaine, "%s (%dp+%dc)",
                                PC_net[nb_PC_net-1].net_nom,
                                PC_net[nb_PC_net-1].nbp,
                                PC_net[nb_PC_net-1].nbc);
                        else
                            sprintf(chaine, "%s (%dh+%do)",
                                PC_net[nb_PC_net-1].net_nom,
                                PC_net[nb_PC_net-1].nbp,
                                PC_net[nb_PC_net-1].nbc);
                        ajoute_texte_preview(-1, chaine);
                        ajoute_texte_preview_nbj();
                        grise_options_menu(0);
                        if (pos_menu == 0)
                            affiche_partie_menu(ECRAN_REEL, 0, pos_advanced, 0);
                        else
                            affiche_partie_menu(ECRAN_REEL, 0, save_advanced[0], 0);
                    }
                }
                else
                {
                    sprintf(message, "SC%c%c%c%c%c%s",
                        (unsigned char)(net_port_local >> 8),
                        (unsigned char)(net_port_local & 255),
                        NETFTRIS_CONNECT_REFUSED,
                        NETFTRIS_REFUS_NBJ_MAX,
                        (unsigned char)(strlen(net_nom)), net_nom);
                    net_driver[net_type].send(adresse_envoyeur, port_envoyeur,
                        7+strlen(net_nom), message);
                }
            }
            break;
        case NETFTRIS_CONNECT_ACCEPTED:
            if (!net_master)
            {
                j = message_recu[5];
                for (i = 0; i < j; i++)
                    nom_serveur[i] = message_recu[6+i];
                nom_serveur[j] = '\0';
                free_texte_preview();
                if (langue == ANGLAIS)
                    sprintf(chaine, "Connected to \"%s\"", nom_serveur);
                else
                    sprintf(chaine, "Connecte a \"%s\"", nom_serveur);
                ajoute_texte_preview(-1, chaine);
                cptr_net_retry = 0;
                net_adresse_serveur = adresse_envoyeur;
                grise_options_menu(3);
                if (pos_menu == 3)
                    affiche_partie_menu(ECRAN_REEL, 3, pos_advanced, 1);
                else
                    affiche_partie_menu(ECRAN_REEL, 3, save_advanced[3], 1);
            }
            break;
        case NETFTRIS_CONNECT_REFUSED:
            if (!net_master)
            {
                j = message_recu[6];
                for (i = 0; i < j; i++)
                    nom_serveur[i] = message_recu[7+i];
                nom_serveur[j] = '\0';
                free_texte_preview();
                nom_serveur[j] = '\0';
                if (langue == ANGLAIS)
                {
                    sprintf(chaine, "Server \"%s\":", nom_serveur);
                    ajoute_texte_preview(-1, chaine);
                    ajoute_texte_preview(-1, "  Connection refused");
                    switch (message_recu[5])
                    {
                        case NETFTRIS_REFUS_NBJ_MAX:
                            ajoute_texte_preview(-1, "  (too much connected PC)");
                            break;
                        case NETFTRIS_REFUS_BOTS:
                            ajoute_texte_preview(-1, "  (server excludes F-Bots)");
                            break;
                        default:
                            ajoute_texte_preview(-1, "  (unknown reason)");
                            break;
                    }
                }
                else
                {
                    sprintf(chaine, "Serveur \"%s\":", nom_serveur);
                    ajoute_texte_preview(-1, chaine);
                    ajoute_texte_preview(-1, "  Connexion refusee");
                    switch (message_recu[5])
                    {
                        case NETFTRIS_REFUS_NBJ_MAX:
                            ajoute_texte_preview(-1, "  (trop de PC connectes)");
                            break;
                        case NETFTRIS_REFUS_BOTS:
                            ajoute_texte_preview(-1, "  (serveur exclue les F-bots)");
                            break;
                        default:
                            ajoute_texte_preview(-1, "  (raison inconnue)");
                            break;
                    }
                }
                annule_reseau(0, 0, 1);
            }
            break;
        case NETFTRIS_DISCONNECT:
            if (en_jeu)
            {
                /* un joueur a quitte la partie */
                for (i = 0; i < nb_PC_net; i++)
                {
                    if ( (PC_net[i].adresse == adresse_envoyeur)
                        && (PC_net[i].port == port_envoyeur) )
                    {
                        PC_net[i].nb_perdus = PC_net[i].nbj;
                        PC_net[i].adresse = 0;
                        PC_net[i].port = 0;
                        verifie_gagne_perdu();
                        break;
                    }
                }
            }
            else
            {
                if (net_master)
                {
                    /* le serveur recoit un disconnect (d'un client) */
                    for (i = 1; i < nb_PC_net; i++)
                    {
                        if ( (PC_net[i].adresse == adresse_envoyeur)
                            && (PC_net[i].port == port_envoyeur) )
                        {
                            for (j = i; j < nb_PC_net-1; j++)
                                memcpy(&PC_net[j], &PC_net[j+1], sizeof(t_PC_net));
                            nb_PC_net--;
                            retire_texte_preview_derniere_ligne();
                            retire_texte_preview_une_ligne(i);
                            ajoute_texte_preview_nbj();
                            break;
                        }
                        grise_options_menu(0);
                        if (pos_menu == 0)
                            affiche_partie_menu(ECRAN_REEL, 0, pos_advanced, 0);
                        else
                            affiche_partie_menu(ECRAN_REEL, 0, save_advanced[0], 0);
                    }
                }
                else
                {
                    /* le client recoit un disconnect (du serveur) */
                    free_texte_preview();
                    if (langue == ANGLAIS)
                        ajoute_texte_preview(-1, "Server canceled the game !");
                    else
                        ajoute_texte_preview(-1, "Le serveur a annule la partie !");
                    annule_reseau(0, 0, 1);
                }
            }
            break;
        case NETFTRIS_START_GAME:
            if (en_jeu)
                ftris_error(MSG_ERROR, BUG, CLOSE_SB16, STOP_PROG,
                    "'Start game' received but game is running !");
            else
            {
                nb_PC_net = (int)(message_recu[5]);
                numero_PC_net = (int)(message_recu[6]);
                j = 7;
                for (i = 0; i < nb_PC_net; i++)
                {
                    PC_net[i].nbp = (int)(message_recu[j++]);
                    PC_net[i].nbc = (int)(message_recu[j++]);
                    PC_net[i].nbj = PC_net[i].nbp+PC_net[i].nbc;
                    PC_net[i].nb_perdus = 0;
                    PC_net[i].adresse = ((int)(message_recu[j]) << 24)+
                        ((int)(message_recu[j+1]) << 16)+
                        ((int)(message_recu[j+2]) << 8)+
                        (int)(message_recu[j+3]);
                    j += 4;
                    PC_net[i].port = ((int)(message_recu[j]) << 8)+
                        (int)(message_recu[j+1]);
                    j += 2;
                    k = (int)(message_recu[j++]);
                    for (l = 0; l < k; l++)
                        PC_net[i].net_nom[l] = message_recu[j++];
                    PC_net[i].net_nom[k] = '\0';
                }
                /* demarrage d'une partie reseau pour un client */
                start_game();
            }
            break;
        case NETFTRIS_LIGNES:
            j = (int)(message_recu[5]);
            for (i = 0; i < nbj; i++)
                doit_remonter[i] += j;
            break;
        case NETFTRIS_PERD:
            i = (int)(message_recu[5]);
            PC_net[i].nb_perdus = PC_net[i].nbj;
            verifie_gagne_perdu();
            break;
    }
}

/*
 * envoi_disconnect: envoie une requete de deconnexion vers un PC du reseau
 */

void envoi_disconnect(unsigned long adresse, int port)
{
    unsigned char   message[MAX_LONG_MSG];

    if (en_jeu)
        sprintf(message, "**%c%c%c",
            (unsigned char)(net_port_local >> 8),
            (unsigned char)(net_port_local & 255),
            NETFTRIS_DISCONNECT);
    else
    {
        if (net_master)
            sprintf(message, "SC%c%c%c",
                (unsigned char)(net_port_local >> 8),
                (unsigned char)(net_port_local & 255),
                NETFTRIS_DISCONNECT);
        else
            sprintf(message, "CS%c%c%c",
                (unsigned char)(net_port_local >> 8),
                (unsigned char)(net_port_local & 255),
                NETFTRIS_DISCONNECT);
    }
    net_driver[net_type].send(adresse, port, 5, message);
}

/*
 * envoi_start_game: envoie l'ordre de demarrer le jeu a tous les clients
 */

void envoi_start_game()
{
    unsigned char   message[MAX_LONG_MSG];
    int             i, j;

    sprintf(message, "SC%c%c%c%c%c",
        (unsigned char)(net_port_local >> 8),
        (unsigned char)(net_port_local & 255),
        NETFTRIS_START_GAME,
        (unsigned char)(nb_PC_net),
        '0' /* change plus bas !*/);
    j = 7;
    for (i = 0; i < nb_PC_net; i++)
    {
        sprintf(&message[j], "%c%c%c%c%c%c%c%c%c",
            (unsigned char)(PC_net[i].nbp),
            (unsigned char)(PC_net[i].nbc),
            (unsigned char)(PC_net[i].adresse >> 24),
            (unsigned char)((PC_net[i].adresse >> 16) & 255),
            (unsigned char)((PC_net[i].adresse >> 8) & 255),
            (unsigned char)(PC_net[i].adresse & 255),
            (unsigned char)(PC_net[i].port >> 8),
            (unsigned char)(PC_net[i].port & 255),
            (unsigned char)(strlen(PC_net[i].net_nom)));
        j += 9;
        sprintf(&message[j], "%s", PC_net[i].net_nom);
        j += strlen(PC_net[i].net_nom);
    }
    for (i = 1; i < nb_PC_net; i++)
    {
        message[6] = (unsigned char)(i);
        net_driver[net_type].send(PC_net[i].adresse, PC_net[i].port,
            j, message);
    }
}

/*
 * envoi_perdu: envoie un message "j'ai perdu" (snif...)
 */

void envoi_perdu()
{
    unsigned char   message[MAX_LONG_MSG];
    int             i;

    sprintf(message, "**%c%c%c%c",
        (unsigned char)(net_port_local >> 8),
        (unsigned char)(net_port_local & 255),
        NETFTRIS_PERD,
        (unsigned char)(numero_PC_net));

    for (i = 0; i < nb_PC_net; i++)
        if ( (i != numero_PC_net) && (PC_net[i].adresse != 0) )
            net_driver[net_type].send(PC_net[i].adresse, PC_net[i].port,
                6, message);
}

/*
 * envoi_lignes: envoi un nombre de lignes donne a tous les joueurs
 */

void envoi_lignes(int nb_lignes)
{
    unsigned char   message[MAX_LONG_MSG];
    int             i;

    sprintf(message, "**%c%c%c%c",
        (unsigned char)(net_port_local >> 8),
        (unsigned char)(net_port_local & 255),
        NETFTRIS_LIGNES,
        (unsigned char)(nb_lignes));

    for (i = 0; i < nb_PC_net; i++)
        if ( (i != numero_PC_net) && (PC_net[i].adresse != 0) )
            net_driver[net_type].send(PC_net[i].adresse, PC_net[i].port,
                6, message);
}
